"""配置模块。

提供所有应用共享的基础配置结构。
使用 pydantic-settings 进行分层分级配置管理。

设计原则：
- Application 层配置完全独立，不依赖 Infrastructure 层
- 配置是纯粹的数据模型定义

多实例配置:
框架支持多种组件的多实例配置，使用统一的环境变量格式:
    {PREFIX}_{INSTANCE}_{FIELD}=value

示例:
    DATABASE_DEFAULT_URL=postgresql://main...
    DATABASE_ANALYTICS_URL=postgresql://analytics...
    CACHE_DEFAULT_URL=redis://localhost:6379/1
    MQ_DEFAULT_URL=redis://localhost:6379/2
"""

from .multi_instance import (
    MultiInstanceConfigLoader,
    MultiInstanceSettings,
    parse_multi_instance_env,
)
from .settings import (
    BaseConfig,
    CacheInstanceConfig,
    CacheSettings,
    ChannelInstanceConfig,
    CORSSettings,
    DatabaseInstanceConfig,
    DatabaseSettings,
    EventInstanceConfig,
    EventSettings,
    HealthCheckSettings,
    LogSettings,
    MessageQueueSettings,
    PrometheusMetricsSettings,
    MigrationSettings,
    MQInstanceConfig,
    RPCClientSettings,
    RPCServiceSettings,
    SchedulerSettings,
    ServerSettings,
    ServiceSettings,
    StorageInstanceConfig,
    TaskSettings,
)

__all__ = [
    # 配置类
    "BaseConfig",
    "CORSSettings",
    # 多实例配置类
    "CacheInstanceConfig",
    "CacheSettings",
    "ChannelInstanceConfig",
    "DatabaseInstanceConfig",
    "DatabaseSettings",
    "EventInstanceConfig",
    "EventSettings",
    "HealthCheckSettings",
    "LogSettings",
    "PrometheusMetricsSettings",
    "MQInstanceConfig",
    "MessageQueueSettings",
    "MigrationSettings",
    # 多实例配置工具
    "MultiInstanceConfigLoader",
    "MultiInstanceSettings",
    "RPCClientSettings",
    "RPCServiceSettings",
    "SchedulerSettings",
    "ServerSettings",
    "ServiceSettings",
    "StorageInstanceConfig",
    "TaskSettings",
    "parse_multi_instance_env",
]




"""自定义 SpanProcessor，用于检测慢 span 和异常 span 并触发告警。"""

from __future__ import annotations

import asyncio
from collections.abc import Awaitable, Callable
import fnmatch
import re
from typing import TYPE_CHECKING, Any

from aury.boot.common.logging import logger

# OTel 可选依赖
try:
    from opentelemetry.trace import SpanKind as OTelSpanKind
    from opentelemetry.trace import StatusCode
except ImportError:
    OTelSpanKind = None  # type: ignore[assignment, misc]
    StatusCode = None  # type: ignore[assignment, misc]

if TYPE_CHECKING:
    from asyncio import Task

    from opentelemetry.sdk.trace import ReadableSpan, Span, SpanProcessor

# 告警回调类型: async def callback(event_type, message, **metadata)
AlertCallback = Callable[[str, str], Awaitable[None]] | Callable[..., Awaitable[None]]


class AlertingSpanProcessor:
    """告警 SpanProcessor。
    
    在 span 结束时检测：
    - 慢 span（duration > threshold）
    - 异常 span（status = ERROR）
    
    通过回调函数触发告警，不直接依赖 alerting 模块。
    
    注意：这是一个同步 SpanProcessor，告警发送会在后台异步执行。
    """
    
    # 保存后台任务引用，避免被 GC 回收
    _background_tasks: set["Task"] = set()
    
    def __init__(
        self,
        *,
        slow_request_threshold: float = 1.0,
        slow_sql_threshold: float = 0.5,
        alert_on_slow_request: bool = True,
        alert_on_slow_sql: bool = True,
        alert_on_error: bool = True,
        alert_callback: AlertCallback | None = None,
        slow_request_exclude_paths: list[str] | None = None,
    ) -> None:
        """初始化 AlertingSpanProcessor。
        
        Args:
            slow_request_threshold: HTTP 请求慢阈值（秒）
            slow_sql_threshold: SQL 查询慢阈值（秒）
            alert_on_slow_request: 是否对慢 HTTP 请求发送告警
            alert_on_slow_sql: 是否对慢 SQL 发送告警
            alert_on_error: 是否对异常 span 发送告警
            alert_callback: 告警回调函数，签名: async (event_type, message, **metadata) -> None
            slow_request_exclude_paths: 慢请求排除路径列表（支持 * 通配符），如 SSE/WebSocket 长连接
        """
        self._slow_request_threshold = slow_request_threshold
        self._slow_sql_threshold = slow_sql_threshold
        self._alert_on_slow_request = alert_on_slow_request
        self._alert_on_slow_sql = alert_on_slow_sql
        self._alert_on_error = alert_on_error
        self._alert_callback = alert_callback
        
        # 编译排除路径正则
        self._exclude_regexes: list[re.Pattern] = []
        if slow_request_exclude_paths:
            for pattern in slow_request_exclude_paths:
                regex_pattern = fnmatch.translate(pattern)
                self._exclude_regexes.append(re.compile(regex_pattern))
    
    def on_start(self, span: "Span", parent_context: object = None) -> None:
        """span 开始时调用（不做处理）。"""
        pass
    
    def on_end(self, span: "ReadableSpan") -> None:
        """span 结束时调用，检测并触发告警。"""
        if StatusCode is None:
            return
        
        # 获取 span 信息
        name = span.name
        duration_ns = (span.end_time or 0) - (span.start_time or 0)
        duration_s = duration_ns / 1e9
        status = span.status
        trace_id = format(span.context.trace_id, "032x") if span.context else ""
        
        # 获取 span 属性
        attributes = dict(span.attributes) if span.attributes else {}
        
        # 判断 span 类型
        span_kind = _get_span_kind(span)
        
        # 根据 span 类型获取对应的慢阈值和开关
        threshold = self._get_slow_threshold(span_kind)
        should_alert = self._should_alert_slow(span_kind)
        
        # 检测慢 span
        if should_alert and threshold > 0 and duration_s >= threshold:
            # 检查是否在排除路径中
            if self._is_path_excluded(name, attributes):
                return
            
            self._emit_slow_alert(
                name=name,
                duration=duration_s,
                trace_id=trace_id,
                span_kind=span_kind,
                attributes=attributes,
                threshold=threshold,
            )
        
        # 检测异常 span（只对 SERVER span 发告警，避免重复）
        if (
            self._alert_on_error
            and status
            and status.status_code == StatusCode.ERROR
            and span_kind == "http"  # 只对 HTTP SERVER span 发异常告警
        ):
            # 过滤 4xx 业务异常（如 401/403/404 等），只对 5xx 系统异常告警
            http_status = attributes.get("http.status_code", 0)
            if 400 <= http_status < 500:
                return  # 业务异常不告警
            
            # 从 span events 中提取异常详情
            exception_info = _extract_exception_info(span)
            self._emit_error_alert(
                name=name,
                duration=duration_s,
                trace_id=trace_id,
                span_kind=span_kind,
                error_message=exception_info.get("message") or status.description or "Unknown error",
                attributes=attributes,
                exception_info=exception_info,
            )
    
    def shutdown(self) -> None:
        """关闭处理器。"""
        pass
    
    def force_flush(self, timeout_millis: int = 30000) -> bool:
        """强制刷新（无缓冲，直接返回）。"""
        return True
    
    def _get_slow_threshold(self, span_kind: str) -> float:
        """根据 span 类型获取对应的慢阈值。"""
        if span_kind == "database":
            return self._slow_sql_threshold
        elif span_kind in ("http", "http_client"):
            return self._slow_request_threshold
        # 其他类型使用 HTTP 阈值作为默认
        return self._slow_request_threshold
    
    def _should_alert_slow(self, span_kind: str) -> bool:
        """根据 span 类型判断是否应该发送慢告警。"""
        if span_kind == "database":
            return self._alert_on_slow_sql
        elif span_kind in ("http", "http_client", "internal"):
            return self._alert_on_slow_request
        # 其他类型默认使用 HTTP 的开关
        return self._alert_on_slow_request
    
    def _is_path_excluded(self, name: str, attributes: dict) -> bool:
        """检查路径是否在排除列表中。"""
        if not self._exclude_regexes:
            return False
        
        # 从 attributes 或 span name 中提取路径
        path = (
            attributes.get("http.route")
            or attributes.get("http.target")
            or name
        )
        
        # 检查所有可能的路径来源
        paths_to_check = [path]
        
        # 也检查 span name 中的路径（可能包含 HTTP 方法和后缀）
        # 例如 "GET /api/v1/spaces/{space_id}/subscribe http receive"
        if name and name != path:
            # 尝试提取 span name 中的路径部分
            parts = name.split()
            for part in parts:
                if part.startswith("/"):
                    paths_to_check.append(part)
        
        for p in paths_to_check:
            if any(regex.fullmatch(p) for regex in self._exclude_regexes):
                return True
        
        return False
    
    def _emit_slow_alert(
        self,
        name: str,
        duration: float,
        trace_id: str,
        span_kind: str,
        attributes: dict,
        threshold: float,
    ) -> None:
        """发送慢 span 告警。"""
        if not self._alert_callback:
            return
        
        try:
            task = asyncio.create_task(
                self._alert_callback(
                    _get_event_type_for_slow(span_kind),
                    f"慢 {span_kind}: {name}",
                    severity="warning",
                    trace_id=trace_id,
                    source=span_kind,
                    duration=duration,
                    threshold=threshold,
                    **_extract_alert_context(attributes),
                )
            )
            self._background_tasks.add(task)
            task.add_done_callback(self._background_tasks.discard)
        except RuntimeError:
            logger.debug(f"无法发送慢 span 告警（无事件循环）: {name}")
    
    def _emit_error_alert(
        self,
        name: str,
        duration: float,
        trace_id: str,
        span_kind: str,
        error_message: str,
        attributes: dict,
        exception_info: dict | None = None,
    ) -> None:
        """发送异常 span 告警。"""
        if not self._alert_callback:
            return
        
        # 合并异常信息
        extra_context = _extract_alert_context(attributes)
        if exception_info:
            if exception_info.get("type"):
                extra_context["error_type"] = exception_info["type"]
            if exception_info.get("stacktrace"):
                extra_context["stacktrace"] = exception_info["stacktrace"]
        
        try:
            task = asyncio.create_task(
                self._alert_callback(
                    "exception",
                    f"异常: {error_message}",
                    severity="error",
                    trace_id=trace_id,
                    source=span_kind,
                    duration=duration,
                    error_message=error_message,
                    **extra_context,
                )
            )
            self._background_tasks.add(task)
            task.add_done_callback(self._background_tasks.discard)
        except RuntimeError:
            logger.debug(f"无法发送异常 span 告警（无事件循环）: {name}")


def _get_span_kind(span: "ReadableSpan") -> str:
    """根据 span 属性推断类型。"""
    if OTelSpanKind is None:
        return "unknown"
    
    kind = span.kind
    
    if kind == OTelSpanKind.SERVER:
        # HTTP 请求
        return "http"
    elif kind == OTelSpanKind.CLIENT:
        # 外部调用（DB、HTTP client 等）
        attributes = dict(span.attributes) if span.attributes else {}
        if "db.system" in attributes:
            return "database"
        elif "http.url" in attributes or "http.target" in attributes:
            return "http_client"
        return "client"
    elif kind == OTelSpanKind.INTERNAL:
        return "internal"
    elif kind == OTelSpanKind.PRODUCER:
        return "producer"
    elif kind == OTelSpanKind.CONSUMER:
        return "consumer"
    
    return "unknown"


def _get_event_type_for_slow(span_kind: str) -> str:
    """根据 span 类型获取慢操作的告警事件类型。"""
    mapping = {
        "http": "slow_request",
        "database": "slow_sql",
        "http_client": "slow_request",
        "internal": "slow_request",  # internal span 也用 slow_request 类型
    }
    return mapping.get(span_kind, "custom")


def _extract_exception_info(span: "ReadableSpan") -> dict:
    """从 span events 中提取异常信息。
    
    OTEL 会将异常作为 span event 记录，包含：
    - exception.type: 异常类型
    - exception.message: 异常消息
    - exception.stacktrace: 堆栈信息
    """
    info = {}
    
    if not span.events:
        return info
    
    for event in span.events:
        if event.name == "exception":
            attrs = dict(event.attributes) if event.attributes else {}
            if "exception.type" in attrs:
                info["type"] = str(attrs["exception.type"])
            if "exception.message" in attrs:
                info["message"] = str(attrs["exception.message"])
            if "exception.stacktrace" in attrs:
                info["stacktrace"] = str(attrs["exception.stacktrace"])
            break  # 只取第一个异常事件
    
    return info


def _extract_alert_context(attributes: dict) -> dict:
    """从 span 属性中提取告警上下文。"""
    context = {}
    
    # HTTP 相关
    if "http.method" in attributes:
        context["method"] = attributes["http.method"]
    if "http.route" in attributes:
        context["route"] = attributes["http.route"]
    if "http.target" in attributes:
        context["endpoint"] = attributes["http.target"]
    if "http.url" in attributes:
        context["url"] = attributes["http.url"]
    if "http.status_code" in attributes:
        context["status_code"] = attributes["http.status_code"]
    
    # 数据库相关
    if "db.system" in attributes:
        context["db_system"] = attributes["db.system"]
    if "db.statement" in attributes:
        context["sql"] = _normalize_sql(str(attributes["db.statement"]))
    
    return context


def _normalize_sql(sql: str) -> str:
    """清理 SQL 多余空白。"""
    import re
    # 将多个空白字符（包括换行、制表符）合并为单个空格
    sql = re.sub(r'\s+', ' ', sql)
    return sql.strip()


__all__ = ["AlertingSpanProcessor"]



"""默认组件实现。

提供所有内置基础设施组件的实现。
"""

from __future__ import annotations

import importlib
import pkgutil
from typing import ClassVar

from aury.boot.application.app.base import Component, FoundationApp, Plugin
from aury.boot.application.config import BaseConfig
from aury.boot.application.constants import ComponentName, ServiceType
from aury.boot.application.migrations import MigrationManager
from aury.boot.common.logging import logger
from aury.boot.infrastructure.cache import CacheManager
from aury.boot.infrastructure.channel import ChannelManager
from aury.boot.infrastructure.database import DatabaseManager, setup_connection_tracking
from aury.boot.infrastructure.events import EventBusManager
from aury.boot.infrastructure.monitoring.alerting import (
    AlertEventType,
    AlertManager,
    AlertSeverity,
    emit_alert,
)
from aury.boot.infrastructure.monitoring.metrics import PrometheusMetricsPlugin
from aury.boot.infrastructure.monitoring.tracing import (
    TelemetryConfig,
    TelemetryProvider,
    setup_otel_logging,
)
from aury.boot.infrastructure.mq import MQManager
from aury.boot.infrastructure.scheduler import SchedulerManager
from aury.boot.infrastructure.storage import StorageManager
from aury.boot.infrastructure.tasks import TaskManager


class DatabaseComponent(Component):
    """数据库组件。"""

    name = ComponentName.DATABASE
    enabled = True
    depends_on: ClassVar[list[str]] = []

    def can_enable(self, config: BaseConfig) -> bool:
        """仅当配置了数据库 URL 时启用。"""
        return self.enabled and bool(config.database.url)

    async def setup(self, app: FoundationApp, config: BaseConfig) -> None:
        """初始化数据库。"""
        try:
            db_manager = DatabaseManager.get_instance()
            if not db_manager._initialized:
                await db_manager.initialize(
                    url=config.database.url,
                    echo=config.database.echo,
                    pool_size=config.database.pool_size,
                    max_overflow=config.database.max_overflow,
                    pool_timeout=config.database.pool_timeout,
                    pool_recycle=config.database.pool_recycle,
                )
                # 在 engine 创建后进行 OTEL instrumentation
                self._instrument_engine(db_manager, config)
        except Exception as e:
            logger.error(f"数据库初始化失败: {e}")
            raise
    
    def _instrument_engine(self, db_manager: DatabaseManager, config: BaseConfig) -> None:
        """对数据库引擎进行 OTEL instrumentation。"""
        if not config.telemetry.enabled:
            return
        
        try:
            from opentelemetry.instrumentation.sqlalchemy import SQLAlchemyInstrumentor
            # async engine 需要使用 sync_engine
            SQLAlchemyInstrumentor().instrument(engine=db_manager.engine.sync_engine)
            logger.debug("SQLAlchemy engine instrumentation 已启用")
        except ImportError:
            logger.debug("SQLAlchemy instrumentation 未安装，跳过")
        except Exception as e:
            logger.warning(f"SQLAlchemy engine instrumentation 失败: {e}")

    async def teardown(self, app: FoundationApp) -> None:
        """关闭数据库。"""
        try:
            db_manager = DatabaseManager.get_instance()
            if db_manager._initialized:
                await db_manager.cleanup()
        except Exception as e:
            logger.error(f"数据库关闭失败: {e}")


class DbConnectionTrackerComponent(Component):
    """数据库连接追踪组件（对标 HikariCP leakDetectionThreshold）。
    
    用于检测连接泄漏，追踪连接的获取来源代码位置。
    
    单实例配置（DATABASE__ 前缀）：
        DATABASE__TRACKER_ENABLED: 是否启用追踪（默认 False）
        DATABASE__LEAK_DETECTION_THRESHOLD: 泄漏检测阈值（秒）
    
    多实例配置（DATABASE__{INSTANCE}__ 前缀）：
        DATABASE__DEFAULT__TRACKER_ENABLED=true
        DATABASE__ANALYTICS__TRACKER_ENABLED=false
    """

    name = "db_connection_tracker"
    enabled = True
    depends_on: ClassVar[list[str]] = [ComponentName.DATABASE]

    def can_enable(self, config: BaseConfig) -> bool:
        """当配置了数据库时启用。"""
        return self.enabled and bool(config.database.url)

    async def setup(self, app: FoundationApp, config: BaseConfig) -> None:
        """启动时注册连接追踪（支持多实例）。"""
        # 从 pyproject.toml 获取应用包名
        try:
            from aury.boot.commands.config import get_project_config
            cfg = get_project_config()
            app_package = cfg.package if cfg.has_package else "app"
        except Exception:
            app_package = "app"
        
        # 为所有已初始化的数据库实例注册追踪
        databases = config.get_databases()
        for name, db_config in databases.items():
            db = DatabaseManager.get_instance(name)
            if db.is_initialized:
                setup_connection_tracking(
                    pool=db.engine.sync_engine.pool,
                    enabled=db_config.tracker_enabled,
                    app_package=app_package,
                )
                if db_config.tracker_enabled:
                    logger.info(f"数据库实例 [{name}] 已启用连接追踪 (threshold={db_config.leak_detection_threshold}s)")

    async def teardown(self, app: FoundationApp) -> None:
        """关闭时无需特殊处理。"""
        pass


class CacheComponent(Component):
    """缓存组件。"""

    name = ComponentName.CACHE
    enabled = True
    depends_on: ClassVar[list[str]] = []

    def can_enable(self, config: BaseConfig) -> bool:
        """仅当配置了缓存类型时启用。"""
        return self.enabled and bool(config.cache.cache_type)

    async def setup(self, app: FoundationApp, config: BaseConfig) -> None:
        """初始化缓存。"""
        try:
            cache_manager = CacheManager.get_instance()
            if not cache_manager.is_initialized:
                await cache_manager.initialize(
                    backend=config.cache.cache_type,
                    url=config.cache.url,
                    max_size=config.cache.max_size,
                )
        except Exception as e:
            logger.warning(f"缓存初始化失败（非关键）: {e}")

    async def teardown(self, app: FoundationApp) -> None:
        """关闭缓存。"""
        try:
            cache_manager = CacheManager.get_instance()
            if cache_manager.is_initialized:
                await cache_manager.cleanup()
        except Exception as e:
            logger.warning(f"缓存关闭失败: {e}")


class StorageComponent(Component):
    """对象存储组件。
    
    支持多实例配置，通过环境变量 STORAGE_{INSTANCE}_{FIELD} 配置。
    """

    name = ComponentName.STORAGE
    enabled = True
    depends_on: ClassVar[list[str]] = []

    def can_enable(self, config: BaseConfig) -> bool:
        """当配置了 Storage 实例时启用。"""
        return self.enabled and bool(config.get_storages())

    async def setup(self, app: FoundationApp, config: BaseConfig) -> None:
        """初始化存储。"""
        from aury.boot.infrastructure.storage import StorageBackend, StorageConfig
        
        storage_configs = config.get_storages()
        if not storage_configs:
            logger.debug("未配置 Storage 实例，跳过存储初始化")
            return
        
        for name, st_config in storage_configs.items():
            try:
                storage_manager = StorageManager.get_instance(name)
                if not storage_manager.is_initialized:
                    storage_config = StorageConfig(
                        backend=StorageBackend(st_config.backend),
                        access_key_id=st_config.access_key_id,
                        access_key_secret=st_config.access_key_secret,
                        endpoint=st_config.endpoint,
                        region=st_config.region,
                        bucket_name=st_config.bucket_name,
                        base_path=st_config.base_path,
                    )
                    await storage_manager.initialize(storage_config)
            except Exception as e:
                logger.warning(f"存储 [{name}] 初始化失败（非关键）: {e}")

    async def teardown(self, app: FoundationApp) -> None:
        """关闭所有存储实例。"""
        for name in list(StorageManager._instances.keys()):
            try:
                storage_manager = StorageManager.get_instance(name)
                if storage_manager.is_initialized:
                    await storage_manager.cleanup()
            except Exception as e:
                logger.warning(f"存储 [{name}] 关闭失败: {e}")


class TaskComponent(Component):
    """任务队列组件（Worker 模式）。"""

    name = ComponentName.TASK_QUEUE
    enabled = True
    depends_on: ClassVar[list[str]] = []

    def can_enable(self, config: BaseConfig) -> bool:
        """仅当是 Worker 模式且配置了 broker URL 时启用。"""
        return (
            self.enabled
            and config.service.service_type == ServiceType.WORKER.value
            and bool(config.task.broker_url)
        )

    async def setup(self, app: FoundationApp, config: BaseConfig) -> None:
        """初始化任务队列。"""
        try:
            from aury.boot.infrastructure.tasks.constants import TaskRunMode
            
            task_manager = TaskManager.get_instance()
            # 将 application 层的 ServiceType.WORKER 转换为 infrastructure 层的 TaskRunMode.WORKER
            # ServiceType.API 和 ServiceType.SCHEDULER 转换为 TaskRunMode.PRODUCER
            if config.service.service_type == ServiceType.WORKER.value:
                run_mode = TaskRunMode.WORKER
            else:
                run_mode = TaskRunMode.PRODUCER
            
            await task_manager.initialize(
                run_mode=run_mode,
                broker_url=config.task.broker_url,
            )
        except Exception as e:
            logger.warning(f"任务队列初始化失败（非关键）: {e}")

    async def teardown(self, app: FoundationApp) -> None:
        """无需显式清理。"""
        pass


class SchedulerComponent(Component):
    """调度器组件。
    
    支持两种任务发现方式：
    1. 配置方式：通过 SCHEDULER_JOB_MODULES 指定要加载的模块
    2. 自动感知：无配置时自动发现 schedules 模块
    
    @scheduler.scheduled_job() 装饰器可以在模块导入时使用，
    任务会被收集到待注册列表，在 start() 时注册并启动。
    """

    name = ComponentName.SCHEDULER
    enabled = True
    depends_on: ClassVar[list[str]] = []

    def can_enable(self, config: BaseConfig) -> bool:
        """仅当是 API 服务且启用调度器时启用。"""
        return (
            self.enabled
            and config.service.service_type == ServiceType.API.value
            and config.scheduler.enabled
        )

    def _autodiscover_schedules(self, app: FoundationApp, config: BaseConfig) -> None:
        """自动发现并加载定时任务模块。
        
        发现策略：
        1. 如果配置了 schedule_modules，直接加载配置的模块
        2. 优先尝试项目包名和服务名：
           - {pyproject.tool.aury.package}.schedules
           - {SERVICE_NAME}.schedules（当 SERVICE_NAME 不是默认值时）
        3. 从 app._caller_module 推断：
           - __main__ / main → 尝试 schedules
           - myapp.main → 尝试 myapp.schedules
        4. 回退：尝试导入 schedules
        """
        modules_to_load: list[str] = []
        
        # 策略 1：配置优先
        if config.scheduler.schedule_modules:
            modules_to_load = list(config.scheduler.schedule_modules)
            logger.debug(f"使用配置的定时任务模块: {modules_to_load}")
        else:
            # 策略 2：项目包名与服务名
            try:
                from aury.boot.commands.config import get_project_config
                cfg = get_project_config()
                if cfg.has_package:
                    modules_to_load.append(f"{cfg.package}.schedules")
            except Exception:
                pass
            
            service_name = (getattr(config.service, "name", None) or "").strip()
            if service_name and service_name not in {"app", "main"}:
                modules_to_load.append(f"{service_name}.schedules")
            
            # 策略 3：从调用者模块推断
            caller = getattr(app, "_caller_module", "__main__")
            if caller in ("__main__", "main"):
                modules_to_load.append("schedules")
            elif "." in caller:
                package = caller.rsplit(".", 1)[0]
                modules_to_load.extend([f"{package}.schedules", "schedules"])
            else:
                modules_to_load.extend([f"{caller}.schedules", "schedules"])
        
        # 去重，保持顺序
        seen = set()
        modules_to_load = [m for m in modules_to_load if not (m in seen or seen.add(m))]
        
        # 加载模块
        for module_name in modules_to_load:
            try:
                module = importlib.import_module(module_name)
                logger.info(f"已自动加载定时任务模块: {module_name}")
                
                # 递归加载包下所有子模块
                if hasattr(module, "__path__"):
                    for _, submodule_name, _ in pkgutil.walk_packages(
                        module.__path__, prefix=f"{module_name}."
                    ):
                        try:
                            importlib.import_module(submodule_name)
                            logger.debug(f"已加载子模块: {submodule_name}")
                        except Exception as e:
                            logger.warning(f"加载子模块失败 ({submodule_name}): {e}")
                
                # 如果成功加载一个，且不是配置模式，就停止
                if not config.scheduler.schedule_modules:
                    break
            except ImportError:
                logger.debug(f"定时任务模块不存在: {module_name}")
            except Exception as e:
                logger.warning(f"加载定时任务模块失败 ({module_name}): {e}")

    def _build_scheduler_config(self, config: BaseConfig) -> dict:
        """根据配置构建 APScheduler 初始化参数。"""
        scheduler_kwargs: dict = {}
        scheduler_config = config.scheduler
        
        # jobstores: 根据 URL 自动选择存储后端
        if scheduler_config.jobstore_url:
            url = scheduler_config.jobstore_url
            if url.startswith("redis-cluster://"):
                # Redis Cluster 模式
                try:
                    from aury.boot.infrastructure.scheduler.jobstores import RedisClusterJobStore
                    
                    scheduler_kwargs["jobstores"] = {
                        "default": RedisClusterJobStore(url=url)
                    }
                    logger.info(f"调度器使用 Redis Cluster 存储: {url.split('@')[-1].split('/')[0]}")
                except ImportError:
                    logger.warning("Redis Cluster jobstore 需要安装 redis[cluster]: pip install 'redis[cluster]'")
            elif url.startswith("redis://"):
                try:
                    from urllib.parse import urlparse
                    from apscheduler.jobstores.redis import RedisJobStore
                    
                    # 解析 Redis URL
                    parsed = urlparse(url)
                    redis_kwargs: dict = {
                        "host": parsed.hostname or "localhost",
                        "port": parsed.port or 6379,
                    }
                    if parsed.password:
                        redis_kwargs["password"] = parsed.password
                    if parsed.path and parsed.path != "/":
                        redis_kwargs["db"] = int(parsed.path.lstrip("/") or 0)
                    
                    scheduler_kwargs["jobstores"] = {
                        "default": RedisJobStore(**redis_kwargs)
                    }
                    logger.info(f"调度器使用 Redis 存储: {parsed.hostname}:{parsed.port}")
                except ImportError:
                    logger.warning("Redis jobstore 需要安装 redis: pip install redis")
            else:
                # SQLAlchemy 存储 (sqlite/postgresql/mysql)
                try:
                    from apscheduler.jobstores.sqlalchemy import SQLAlchemyJobStore
                    scheduler_kwargs["jobstores"] = {
                        "default": SQLAlchemyJobStore(url=url)
                    }
                    logger.info("调度器使用 SQLAlchemy 存储")
                except ImportError:
                    logger.warning("SQLAlchemy jobstore 需要安装 sqlalchemy")
        
        # timezone
        if scheduler_config.timezone:
            scheduler_kwargs["timezone"] = scheduler_config.timezone
        
        # job_defaults
        scheduler_kwargs["job_defaults"] = {
            "coalesce": scheduler_config.coalesce,
            "max_instances": scheduler_config.max_instances,
            "misfire_grace_time": scheduler_config.misfire_grace_time,
        }
        
        return scheduler_kwargs
    
    async def setup(self, app: FoundationApp, config: BaseConfig) -> None:
        """启动调度器。
        
        1. 根据配置初始化调度器（jobstore/timezone/job_defaults）
        2. 自动发现并加载定时任务模块
        3. 启动调度器（注册装饰器收集的任务）
        """
        try:
            # 构建配置
            scheduler_kwargs = self._build_scheduler_config(config)
            
            # 获取/创建调度器实例
            scheduler = SchedulerManager.get_instance("default", **scheduler_kwargs)
            
            # 自动发现并加载定时任务模块
            self._autodiscover_schedules(app, config)
            
            # 启动调度器
            scheduler.start()
        except Exception as e:
            logger.warning(f"调度器启动失败（非关键）: {e}")

    async def teardown(self, app: FoundationApp) -> None:
        """关闭调度器。"""
        try:
            scheduler = SchedulerManager.get_instance()
            if scheduler._scheduler and scheduler._scheduler.running:
                scheduler.shutdown()
        except Exception as e:
            logger.warning(f"调度器关闭失败: {e}")


class MigrationComponent(Component):
    """数据库迁移组件。
    
    自动执行数据库迁移（升级到最新版本）。
    
    配置选项：
    - `ENABLE_AUTO_MIGRATION`：是否启用自动迁移（默认：True）
    - `ALEMBIC_CONFIG_PATH`：Alembic 配置文件路径（默认：alembic.ini）
    - `AUTO_MIGRATE_ON_STARTUP`：应用启动时是否自动执行迁移（默认：True）
    
    使用示例：
        # 在应用启动时自动执行迁移
        app = FoundationApp()
        # MigrationComponent 会在 DatabaseComponent 之后自动执行迁移
    """

    name = ComponentName.MIGRATIONS
    enabled = True
    depends_on: ClassVar[list[str]] = [ComponentName.DATABASE]

    def can_enable(self, config: BaseConfig) -> bool:
        """仅当配置了数据库 URL 时启用。"""
        return self.enabled and bool(config.database.url)

    async def setup(self, app: FoundationApp, config: BaseConfig) -> None:
        """执行数据库迁移。
        
        在应用启动时自动执行所有待处理的迁移，升级数据库到最新版本。
        """
        try:
            # 创建迁移管理器（从配置读取参数）
            migration_settings = config.migration
            migration_manager = MigrationManager(
                database_url=config.database.url,
                config_path=migration_settings.config_path,
                script_location=migration_settings.script_location,
                model_modules=migration_settings.model_modules,
                auto_create=migration_settings.auto_create,
            )
            
            # 检查是否有迁移需要执行
            logger.info("🔄 检查数据库迁移...")
            status = await migration_manager.status()
            
            pending = status.get("pending", [])
            applied = status.get("applied", [])
            
            if pending:
                logger.info("📊 数据库迁移状态：")
                logger.info(f"   已执行: {len(applied)} 个迁移")
                logger.info(f"   待执行: {len(pending)} 个迁移")
                
                # 执行迁移到最新版本
                logger.info("⏳ 执行数据库迁移...")
                await migration_manager.upgrade(revision="head")
                
                logger.info("✅ 数据库迁移完成")
            else:
                logger.info("✅ 数据库已是最新版本，无需迁移")
        except Exception as e:
            logger.error(f"❌ 数据库迁移失败: {e}", exc_info=True)
            raise

    async def teardown(self, app: FoundationApp) -> None:
        """无需清理。"""
        pass


class AdminConsoleComponent(Component):
    """管理后台组件（SQLAdmin Admin Console）。

    配置选项（环境变量前缀 ADMIN_ / ADMIN_AUTH_）：
    - ADMIN_ENABLED: 是否启用（默认 False）
    - ADMIN_PATH: 后台路径（默认 /api/admin-console）
    - ADMIN_DATABASE_URL: 同步数据库 URL（可选）
    - ADMIN_AUTH_MODE: basic/bearer/none/custom/jwt
    - ADMIN_AUTH_*: 认证参数

    注意：sqladmin 通常要求同步 SQLAlchemy Engine。
    """

    name = ComponentName.ADMIN_CONSOLE
    enabled = True
    depends_on: ClassVar[list[str]] = [ComponentName.DATABASE]

    def can_enable(self, config: BaseConfig) -> bool:
        return self.enabled and bool(getattr(getattr(config, "admin", None), "enabled", False))

    async def setup(self, app: FoundationApp, config: BaseConfig) -> None:
        try:
            from aury.boot.contrib.admin_console import install_admin_console

            install_admin_console(app, config)
        except ImportError as e:
            logger.error(f"管理后台启用失败：缺少依赖（请安装 aury-boot[admin]）: {e}")
            raise
        except Exception as e:
            logger.error(f"管理后台初始化失败: {e}")
            raise

    async def teardown(self, app: FoundationApp) -> None:
        # SQLAdmin 路由挂载后无需额外 teardown
        pass


class MessageQueueComponent(Component):
    """消息队列组件。
    
    提供统一的消息队列接口，支持多种后端（Redis、RabbitMQ）。
    与 TaskComponent（基于 Dramatiq）的区别：
    - Task: 异步任务处理（API 发送，Worker 执行）
    - MQ: 通用消息队列（生产者/消费者模式，服务间通信）
    """

    name = ComponentName.MESSAGE_QUEUE
    enabled = True
    depends_on: ClassVar[list[str]] = []

    def can_enable(self, config: BaseConfig) -> bool:
        """当配置了 MQ 实例时启用。"""
        return self.enabled and bool(config.get_mqs())

    async def setup(self, app: FoundationApp, config: BaseConfig) -> None:
        """初始化消息队列。"""
        from aury.boot.application.config import MQInstanceConfig
        
        # 从多实例配置加载
        mq_configs = config.get_mqs()
        if not mq_configs:
            logger.debug("未配置 MQ 实例，跳过消息队列初始化")
            return
        
        for name, mq_config in mq_configs.items():
            try:
                mq_manager = MQManager.get_instance(name)
                if not mq_manager.is_initialized:
                    await mq_manager.initialize(config=mq_config)
            except Exception as e:
                logger.warning(f"消息队列 [{name}] 初始化失败（非关键）: {e}")

    async def teardown(self, app: FoundationApp) -> None:
        """关闭所有消息队列实例。"""
        for name in list(MQManager._instances.keys()):
            try:
                mq_manager = MQManager.get_instance(name)
                if mq_manager.is_initialized:
                    await mq_manager.cleanup()
            except Exception as e:
                logger.warning(f"消息队列 [{name}] 关闭失败: {e}")


class ChannelComponent(Component):
    """流式通道组件。
    
    用于 SSE（Server-Sent Events）和实时通信场景，支持 memory 和 redis 后端。
    """

    name = ComponentName.CHANNEL
    enabled = True
    depends_on: ClassVar[list[str]] = []

    def can_enable(self, config: BaseConfig) -> bool:
        """当配置了 Channel 实例时启用。"""
        return self.enabled and bool(config.get_channels())

    async def setup(self, app: FoundationApp, config: BaseConfig) -> None:
        """初始化流式通道。"""
        channel_configs = config.get_channels()
        if not channel_configs:
            logger.debug("未配置 Channel 实例，跳过通道初始化")
            return
        
        for name, ch_config in channel_configs.items():
            try:
                channel_manager = ChannelManager.get_instance(name)
                if not channel_manager.is_initialized:
                    await channel_manager.initialize(
                        backend=ch_config.backend,
                        url=ch_config.url,
                    )
            except Exception as e:
                logger.warning(f"通道 [{name}] 初始化失败（非关键）: {e}")

    async def teardown(self, app: FoundationApp) -> None:
        """关闭所有通道实例。"""
        for name in list(ChannelManager._instances.keys()):
            try:
                channel_manager = ChannelManager.get_instance(name)
                if channel_manager.is_initialized:
                    await channel_manager.cleanup()
            except Exception as e:
                logger.warning(f"通道 [{name}] 关闭失败: {e}")


class TelemetryPlugin(Plugin):
    """OpenTelemetry 插件。
    
    在 app 创建后同步执行，负责：
    - 初始化 TracerProvider
    - 对 FastAPI app 进行 instrumentation
    - 配置 AlertingSpanProcessor 检测慢请求/异常（告警配置从 ALERT__ 读取）
    
    配置方式（环境变量）：
        TELEMETRY__ENABLED=true
        TELEMETRY__TRACES_ENDPOINT=http://jaeger:4317  # 可选
        TELEMETRY__LOGS_ENDPOINT=http://loki:3100      # 可选
        TELEMETRY__METRICS_ENDPOINT=http://prometheus:9090  # 可选
        
        # 告警相关配置从 ALERT__ 读取
        ALERT__ENABLED=true
        ALERT__SLOW_REQUEST_THRESHOLD=1.0
        ALERT__SLOW_SQL_THRESHOLD=0.5
        ALERT__ALERT_ON_SLOW_REQUEST=true
        ALERT__ALERT_ON_SLOW_SQL=true
        ALERT__ALERT_ON_ERROR=true
    """

    name = "telemetry"
    enabled = True

    def can_enable(self, config: BaseConfig) -> bool:
        """仅当 TELEMETRY__ENABLED=true 时启用。"""
        return self.enabled and config.telemetry.enabled

    def install(self, app: FoundationApp, config: BaseConfig) -> None:
        """初始化 OpenTelemetry 并对 app 进行 instrumentation。"""
        try:
            # 创建告警回调（调用 alerting 模块，自动应用定制化规则）
            alert_callback = self._create_alert_callback() if config.alert.enabled else None
            
            telemetry_config = TelemetryConfig(
                service_name=config.service.name,
                service_version=config.service.version,
                environment=config.service.environment,
                instrument_fastapi=config.telemetry.instrument_fastapi,
                instrument_sqlalchemy=config.telemetry.instrument_sqlalchemy,
                instrument_httpx=config.telemetry.instrument_httpx,
                # 告警配置全部从 AlertSettings 读取
                alert_enabled=config.alert.enabled,
                slow_request_threshold=config.alert.slow_request_threshold,
                slow_sql_threshold=config.alert.slow_sql_threshold,
                alert_on_slow_request=config.alert.alert_on_slow_request,
                alert_on_slow_sql=config.alert.alert_on_slow_sql,
                alert_on_error=config.alert.alert_on_error,
                alert_callback=alert_callback,
                slow_request_exclude_paths=config.alert.slow_request_exclude_paths,
                traces_endpoint=config.telemetry.traces_endpoint,
                traces_headers=config.telemetry.traces_headers,
                logs_endpoint=config.telemetry.logs_endpoint,
                logs_headers=config.telemetry.logs_headers,
                metrics_endpoint=config.telemetry.metrics_endpoint,
                metrics_headers=config.telemetry.metrics_headers,
                sampling_rate=config.telemetry.sampling_rate,
            )
            
            # 创建并初始化 TelemetryProvider
            provider = TelemetryProvider(telemetry_config)
            success = provider.initialize()
            
            if success:
                # 保存到 app.state 以便 shutdown 时使用
                app.state.telemetry_provider = provider
                
                # 对已创建的 FastAPI app 进行 instrumentation
                provider.instrument_fastapi_app(app)
                
                # 配置 OTLP 日志导出（可选）
                if config.telemetry.logs_endpoint:
                    self._setup_otlp_logging(
                        config.telemetry.logs_endpoint,
                        config.telemetry.logs_headers,
                    )
            else:
                logger.warning("OpenTelemetry 初始化失败（非关键）")
                
        except ImportError as e:
            logger.warning(f"OpenTelemetry 未安装，跳过初始化: {e}")
        except Exception as e:
            logger.warning(f"OpenTelemetry 初始化失败（非关键）: {e}")

    def _create_alert_callback(self):
        """创建告警回调函数。
        
        回调会调用 alerting 模块的 emit_alert，自动应用 YAML 定制化规则。
        """
        # 事件类型映射
        type_mapping = {
            "slow_request": AlertEventType.SLOW_REQUEST,
            "slow_sql": AlertEventType.SLOW_SQL,
            "exception": AlertEventType.EXCEPTION,
            "custom": AlertEventType.CUSTOM,
        }
        
        # 严重级别映射
        severity_mapping = {
            "info": AlertSeverity.INFO,
            "warning": AlertSeverity.WARNING,
            "error": AlertSeverity.ERROR,
            "critical": AlertSeverity.CRITICAL,
        }
        
        async def callback(event_type: str, message: str, **metadata):
            try:
                alert_event_type = type_mapping.get(event_type, AlertEventType.CUSTOM)
                severity_str = metadata.pop("severity", "warning")
                severity = severity_mapping.get(severity_str, AlertSeverity.WARNING)
                
                await emit_alert(
                    alert_event_type,
                    message,
                    severity=severity,
                    **metadata,
                )
            except Exception as e:
                logger.debug(f"发送告警失败: {e}")
        
        return callback
    
    def _setup_otlp_logging(
        self,
        endpoint: str,
        headers: dict[str, str] | None = None,
    ) -> None:
        """配置 OTLP 日志导出。"""
        try:
            setup_otel_logging(endpoint=endpoint, headers=headers)
        except Exception as e:
            logger.warning(f"OTLP 日志导出配置失败: {e}")

    def uninstall(self, app: FoundationApp) -> None:
        """关闭 OpenTelemetry。"""
        try:
            provider = getattr(app.state, "telemetry_provider", None)
            if provider:
                provider.shutdown()
                logger.info("OpenTelemetry 已关闭")
        except Exception as e:
            logger.warning(f"OpenTelemetry 关闭失败: {e}")


class AlertComponent(Component):
    """告警系统组件。
    
    在 lifespan 启动时异步初始化 AlertManager。
    """

    name = "alert"
    enabled = True
    depends_on: ClassVar[list[str]] = []

    def can_enable(self, config: BaseConfig) -> bool:
        """仅当 ALERT__ENABLED=true 时启用。"""
        return self.enabled and config.alert.enabled

    async def setup(self, app: FoundationApp, config: BaseConfig) -> None:
        """初始化告警管理器。"""
        try:
            alert_manager = AlertManager.get_instance()
            if not alert_manager.is_initialized:
                await alert_manager.initialize(
                    enabled=config.alert.enabled,
                    service_name=config.service.name,
                    rules_file=config.alert.rules_file,
                    defaults={
                        "slow_request_threshold": config.alert.slow_request_threshold,
                        "slow_sql_threshold": config.alert.slow_sql_threshold,
                        "aggregate_window": config.alert.aggregate_window,
                        "slow_request_aggregate": config.alert.slow_request_aggregate,
                        "slow_sql_aggregate": config.alert.slow_sql_aggregate,
                        "exception_aggregate": config.alert.exception_aggregate,
                        "suppress_seconds": config.alert.suppress_seconds,
                        "slow_request_exclude_paths": config.alert.slow_request_exclude_paths,
                    },
                    notifiers=config.alert.get_notifiers(),
                )
        except Exception as e:
            logger.warning(f"告警管理器初始化失败（非关键）: {e}")

    async def teardown(self, app: FoundationApp) -> None:
        """无需清理。"""
        pass


class ProfilingComponent(Component):
    """Profiling 组件。
    
    提供持续性能分析和事件循环阻塞检测：
    - Pyroscope：持续采样生成火焰图
    - 阻塞检测：检测同步代码阻塞事件循环
    """

    name = "profiling"
    enabled = True
    depends_on: ClassVar[list[str]] = ["alert"]  # 告警依赖

    def can_enable(self, config: BaseConfig) -> bool:
        """当启用 Pyroscope 或阻塞检测时启用。"""
        return self.enabled and (
            config.profiling.enabled or config.profiling.blocking_detector_enabled
        )

    async def setup(self, app: FoundationApp, config: BaseConfig) -> None:
        """初始化 Profiling 组件。"""
        import gc
        
        # 应用 GC 阈值配置
        gc_thresholds = (
            config.profiling.gc_threshold_gen0,
            config.profiling.gc_threshold_gen1,
            config.profiling.gc_threshold_gen2,
        )
        default_thresholds = (700, 10, 10)
        if gc_thresholds != default_thresholds:
            gc.set_threshold(*gc_thresholds)
            logger.info(f"GC 阈值已调整: {gc_thresholds}")
        
        try:
            from aury.boot.infrastructure.monitoring.profiling import (
                ProfilingConfig,
                ProfilingManager,
            )
            
            profiling_config = ProfilingConfig(
                enabled=config.profiling.enabled,
                pyroscope_endpoint=config.profiling.pyroscope_endpoint,
                pyroscope_auth_token=config.profiling.pyroscope_auth_token,
                service_name=config.service.name,
                environment=config.service.environment,
                blocking_detector_enabled=config.profiling.blocking_detector_enabled,
                blocking_check_interval_ms=config.profiling.blocking_check_interval_ms,
                blocking_threshold_ms=config.profiling.blocking_threshold_ms,
                blocking_severe_threshold_ms=config.profiling.blocking_severe_threshold_ms,
                blocking_alert_enabled=config.profiling.blocking_alert_enabled,
                blocking_alert_cooldown_seconds=config.profiling.blocking_alert_cooldown_seconds,
                blocking_max_history=config.profiling.blocking_max_history,
                tags=config.profiling.pyroscope_tags,
            )
            
            manager = ProfilingManager.get_instance()
            manager.configure(profiling_config)
            await manager.start()
            
            # 保存到 app.state
            app.state.profiling_manager = manager
            
        except ImportError as e:
            logger.warning(f"Profiling 依赖未安装，跳过初始化: {e}")
        except Exception as e:
            logger.warning(f"Profiling 初始化失败（非关键）: {e}")

    async def teardown(self, app: FoundationApp) -> None:
        """停止 Profiling 组件。"""
        try:
            manager = getattr(app.state, "profiling_manager", None)
            if manager:
                await manager.stop()
        except Exception as e:
            logger.warning(f"Profiling 关闭失败: {e}")


class EventBusComponent(Component):
    """事件总线组件。
    
    提供发布/订阅模式的事件总线功能，支持多种后端（memory、Redis、RabbitMQ）。
    """

    name = ComponentName.EVENT_BUS
    enabled = True
    depends_on: ClassVar[list[str]] = []

    def can_enable(self, config: BaseConfig) -> bool:
        """当配置了 Event 实例时启用。"""
        return self.enabled and bool(config.get_events())

    async def setup(self, app: FoundationApp, config: BaseConfig) -> None:
        """初始化事件总线。"""
        # 从多实例配置加载
        event_configs = config.get_events()
        if not event_configs:
            logger.debug("未配置 Event 实例，跳过事件总线初始化")
            return
        
        for name, event_config in event_configs.items():
            try:
                event_manager = EventBusManager.get_instance(name)
                if not event_manager.is_initialized:
                    await event_manager.initialize(config=event_config)
            except Exception as e:
                logger.warning(f"事件总线 [{name}] 初始化失败（非关键）: {e}")

    async def teardown(self, app: FoundationApp) -> None:
        """关闭所有事件总线实例。"""
        for name in list(EventBusManager._instances.keys()):
            try:
                event_manager = EventBusManager.get_instance(name)
                if event_manager.is_initialized:
                    await event_manager.cleanup()
            except Exception as e:
                logger.warning(f"事件总线 [{name}] 关闭失败: {e}")


# 设置默认插件
FoundationApp.plugins = [
    TelemetryPlugin,  # app 创建后立即 instrument
    PrometheusMetricsPlugin,  # Prometheus Metrics
]

# 设置默认组件
FoundationApp.components = [
    AlertComponent,  # 最先初始化告警管理器
    ProfilingComponent,  # Profiling 依赖告警
    DatabaseComponent,
    DbConnectionTrackerComponent,  # 连接追踪（依赖 Database）
    MigrationComponent,
    AdminConsoleComponent,
    CacheComponent,
    StorageComponent,
    TaskComponent,
    MessageQueueComponent,
    ChannelComponent,
    EventBusComponent,
    SchedulerComponent,
]


__all__ = [
    "AdminConsoleComponent",
    "AlertComponent",
    "CacheComponent",
    "ChannelComponent",
    "DatabaseComponent",
    "DbConnectionTrackerComponent",
    "EventBusComponent",
    "MessageQueueComponent",
    "PrometheusMetricsPlugin",
    "MigrationComponent",
    "ProfilingComponent",
    "SchedulerComponent",
    "StorageComponent",
    "TaskComponent",
    "TelemetryPlugin",
]




"""流式通道模块。

提供发布/订阅模式的通道功能，用于 SSE、WebSocket 等实时通信场景。

支持的后端（通过 Broadcaster 库）:
- memory:// - 内存通道（单进程，开发/测试用）
- redis:// - Redis Pub/Sub（多进程/分布式）
- redis-cluster:// - Redis Cluster（自动转换为普通 Pub/Sub）
- kafka:// - Apache Kafka
- postgres:// - PostgreSQL LISTEN/NOTIFY
"""

from .backends import BroadcasterChannel
from .base import ChannelBackend, ChannelMessage, IChannel
from .manager import ChannelManager

__all__ = [
    # 接口和类型
    "ChannelBackend",
    "ChannelMessage",
    "IChannel",
    # 管理器
    "ChannelManager",
    # 后端实现
    "BroadcasterChannel",
]



"""调度器启动器。

提供调度器独立进程的启动入口。
"""

from __future__ import annotations

import asyncio
from collections.abc import Awaitable, Callable
import signal

from aury.boot.application.config import BaseConfig
from aury.boot.application.constants import ServiceType
from aury.boot.common.logging import logger
from aury.boot.infrastructure.scheduler import SchedulerManager


async def run_scheduler(
    config: BaseConfig | None = None,
    *,
    register_jobs: Callable[[], None] | Callable[[], Awaitable[None]] | None = None,
) -> None:
    """运行调度器进程（独立模式）。
    
    当 SERVICE_TYPE=scheduler 时，使用此函数启动独立的调度器进程。
    
    Args:
        config: 应用配置（可选，默认从环境变量加载）
        register_jobs: 注册任务的回调函数（可选）
    
    使用示例:
        async def setup_jobs():
            scheduler = SchedulerManager.get_instance()
            scheduler.add_job(
                func=my_task,
                trigger="interval",
                seconds=60
            )
        
        if __name__ == "__main__":
            asyncio.run(run_scheduler(register_jobs=setup_jobs))
    """
    # 加载配置
    if config is None:
        config = BaseConfig()
    
    service_config = config.service
    
    # 验证服务类型
    try:
        service_type = ServiceType(service_config.service_type.lower())
    except ValueError as e:
        raise ValueError(f"无效的服务类型: {service_config.service_type}") from e
    
    if service_type != ServiceType.SCHEDULER:
        raise ValueError(
            f"此函数仅用于 SERVICE_TYPE=scheduler，当前类型: {service_config.service_type}"
        )
    
    logger.info("启动调度器进程（独立模式）")
    
    # 初始化调度器
    scheduler_manager = SchedulerManager.get_instance()
    await scheduler_manager.initialize()
    
    # 注册任务
    if register_jobs:
        logger.info("注册定时任务...")
        if asyncio.iscoroutinefunction(register_jobs):
            await register_jobs()
        else:
            register_jobs()
        jobs = scheduler_manager.get_jobs()
        logger.info(f"已注册 {len(jobs)} 个定时任务")
    else:
        logger.warning("未提供任务注册函数，调度器将不执行任何任务")
    
    # 启动调度器
    scheduler_manager.start()
    logger.info("调度器进程已启动，等待任务执行...")
    
    # 设置信号处理，优雅关闭
    def signal_handler(signum, frame):
        logger.info(f"收到信号 {signum}，正在关闭调度器...")
        scheduler_manager.shutdown()
        logger.info("调度器已关闭")
        asyncio.get_event_loop().stop()
    
    signal.signal(signal.SIGINT, signal_handler)
    signal.signal(signal.SIGTERM, signal_handler)
    
    # 保持进程运行
    try:
        # 使用 asyncio.Event 保持进程运行
        stop_event = asyncio.Event()
        await stop_event.wait()
    except KeyboardInterrupt:
        logger.info("收到中断信号，正在关闭...")
        scheduler_manager.shutdown()
        logger.info("调度器已关闭")


def run_scheduler_sync(
    config: BaseConfig | None = None,
    *,
    register_jobs: Callable[[], None] | Callable[[], Awaitable[None]] | None = None,
) -> None:
    """运行调度器进程（同步版本）。
    
    这是 run_scheduler 的同步包装，方便直接调用。
    
    Args:
        config: 应用配置（可选，默认从环境变量加载）
        register_jobs: 注册任务的回调函数（可选）
    """
    asyncio.run(run_scheduler(config=config, register_jobs=register_jobs))


__all__ = [
    "run_scheduler",
    "run_scheduler_sync",
]




"""存储工厂 - 注册机制。

通过注册机制支持多种存储后端。
"""

from __future__ import annotations

from typing import ClassVar

from aury.boot.common.logging import logger
from aury.sdk.storage.storage import IStorage, LocalStorage


class StorageFactory:
    """存储工厂 - 注册机制。
    
    类似缓存工厂的设计，通过注册机制支持多种后端。
    
    使用示例:
        # 注册后端
        StorageFactory.register("local", LocalStorage)
        StorageFactory.register("s3", S3Storage)
        
        # 创建存储实例
        storage = await StorageFactory.create("local", base_path="./storage")
        storage = await StorageFactory.create("s3", access_key_id="...", ...)
    """
    
    _backends: ClassVar[dict[str, type[IStorage]]] = {}
    
    @classmethod
    def register(cls, name: str, backend_class: type[IStorage]) -> None:
        """注册存储后端。
        
        Args:
            name: 后端名称
            backend_class: 后端类
        """
        cls._backends[name] = backend_class
        logger.debug(f"注册存储后端: {name} -> {backend_class.__name__}")
    
    @classmethod
    async def create(cls, backend_name: str, **config) -> IStorage:
        """创建存储实例。
        
        Args:
            backend_name: 后端名称
            **config: 配置参数
            
        Returns:
            IStorage: 存储实例
            
        Raises:
            ValueError: 后端未注册
        """
        if backend_name not in cls._backends:
            available = ", ".join(cls._backends.keys())
            raise ValueError(
                f"存储后端 '{backend_name}' 未注册。"
                f"可用后端: {available}"
            )
        
        backend_class = cls._backends[backend_name]
        instance = backend_class(**config)
        
        # 如果后端需要初始化
        if hasattr(instance, "initialize"):
            await instance.initialize()
        
        logger.info(f"创建存储实例: {backend_name}")
        return instance
    
    @classmethod
    def get_registered(cls) -> list[str]:
        """获取已注册的后端名称。"""
        return list(cls._backends.keys())


# 注册默认后端（本地）
StorageFactory.register("local", LocalStorage)
# 说明：S3/COS/OSS 请直接使用 StorageManager + StorageConfig 或 SDK 中的 S3Storage


__all__ = [
    "StorageFactory",
]




"""第三方接口适配器基类。

本模块用于封装第三方接口（如支付、短信、微信、云存储等外部服务）的调用，
提供统一的 Adapter 抽象，只约束"模式怎么切换"，不限制具体调用方式。
支持 HTTP、SDK、gRPC 等任意协议。

核心职责：
- 模式路由：real(生产)/sandbox(沙箱)/mock(挡板)/disabled(禁用)
- 调用历史记录：记录每次调用的参数、结果、耗时，便于测试断言
- 挡板调用分发：自动根据配置切换真实调用与挡板实现
- 生命周期钩子：before_call、after_call、on_error 等

挡板（Mock）配置说明：
1. 全局挡板：通过 settings.mode="mock" 设置整个 Adapter 使用挡板
2. 方法级挡板：通过 settings.method_modes={"create": "mock", "query": "real"}
   可以让不同方法使用不同模式（如查询走真实，写入走挡板）
3. 挡板实现：使用 @adapter_method("name") + @method.mock 装饰器定义挡板逻辑
4. 固定响应：简单场景可直接配置 mock_response 或 mock_method_responses

设计原则：
- 最小约束：只管模式路由，不限制调用方式
- 可扩展：通过钩子方法支持自定义行为
- 可观测：记录所有调用历史
"""

from __future__ import annotations

import asyncio
from dataclasses import dataclass, field
from datetime import datetime
from typing import TYPE_CHECKING, Any

from aury.boot.common.logging import logger

from .config import AdapterMode, AdapterSettings

if TYPE_CHECKING:
    from .decorators import AdapterMethodMeta


@dataclass
class CallRecord:
    """调用记录。
    
    记录每次 Adapter 调用的详细信息，用于测试断言和调试。
    
    Attributes:
        operation: 操作名称
        args: 位置参数
        kwargs: 关键字参数
        mode: 调用时的模式
        timestamp: 调用时间戳
        result: 调用结果（如果成功）
        error: 异常信息（如果失败）
        duration_ms: 调用耗时（毫秒）
    """
    
    operation: str
    args: tuple[Any, ...]
    kwargs: dict[str, Any]
    mode: AdapterMode
    timestamp: datetime = field(default_factory=datetime.now)
    result: Any = None
    error: str | None = None
    duration_ms: float | None = None
    
    def to_dict(self) -> dict[str, Any]:
        """转换为字典。"""
        return {
            "operation": self.operation,
            "args": self.args,
            "kwargs": self.kwargs,
            "mode": self.mode,
            "timestamp": self.timestamp.isoformat(),
            "result": self.result,
            "error": self.error,
            "duration_ms": self.duration_ms,
        }


class BaseAdapter:
    """第三方集成 Adapter 基类。
    
    封装模式路由和调用记录逻辑，子类只需实现具体的调用方法。
    
    核心功能：
    - 根据 settings.mode 和 operation_modes 自动路由
    - 支持 @adapter_method 装饰器声明的 mock 逻辑
    - 记录调用历史，便于测试断言
    - 提供生命周期钩子（before_call, after_call, on_error）
    
    使用示例:
        class PaymentAdapter(BaseAdapter):
            @adapter_method("create")
            async def create_order(self, amount: int) -> dict:
                # 这里是真实调用逻辑
                return await some_sdk.create(amount=amount)
            
            @create_order.mock
            async def create_order_mock(self, amount: int) -> dict:
                # mock 逻辑
                return {"success": True, "mock": True}
        
        # 使用
        settings = AdapterSettings(mode="mock")
        adapter = PaymentAdapter("payment", settings)
        result = await adapter.create_order(100)  # 走 mock
        
        # 测试断言
        assert len(adapter.call_history) == 1
        assert adapter.call_history[0].mode == "mock"
    
    Attributes:
        name: Adapter 名称（用于日志和异常信息）
        settings: 集成配置
    """
    
    def __init__(
        self,
        name: str,
        settings: AdapterSettings,
    ) -> None:
        """初始化 Adapter。
        
        Args:
            name: Adapter 名称
            settings: 集成配置
        """
        self.name = name
        self.settings = settings
        self._call_history: list[CallRecord] = []
        self._initialized = False
        
        logger.debug(
            f"初始化 Adapter: {name} | "
            f"mode={settings.mode} | "
            f"enabled={settings.enabled}"
        )
    
    # ========== 公共 API ==========
    
    @property
    def call_history(self) -> list[CallRecord]:
        """获取调用历史（只读）。
        
        Returns:
            list[CallRecord]: 调用记录列表
        """
        return list(self._call_history)
    
    def clear_history(self) -> None:
        """清空调用历史。
        
        通常在测试的 setUp 或 tearDown 中使用。
        """
        self._call_history.clear()
    
    def get_calls_by_operation(self, operation: str) -> list[CallRecord]:
        """获取指定操作的调用记录。
        
        Args:
            operation: 操作名称
            
        Returns:
            list[CallRecord]: 符合条件的调用记录
        """
        return [c for c in self._call_history if c.operation == operation]
    
    def get_last_call(self, operation: str | None = None) -> CallRecord | None:
        """获取最后一次调用记录。
        
        Args:
            operation: 操作名称（可选，不指定则返回最后一次任意调用）
            
        Returns:
            CallRecord | None: 最后一次调用记录
        """
        if operation:
            calls = self.get_calls_by_operation(operation)
            return calls[-1] if calls else None
        return self._call_history[-1] if self._call_history else None
    
    def assert_called(self, operation: str, times: int | None = None) -> None:
        """断言操作被调用（用于测试）。
        
        Args:
            operation: 操作名称
            times: 预期调用次数（可选，不指定则只检查是否被调用过）
            
        Raises:
            AssertionError: 断言失败
        """
        calls = self.get_calls_by_operation(operation)
        if times is not None:
            assert len(calls) == times, (
                f"Expected {operation} to be called {times} times, "
                f"but was called {len(calls)} times"
            )
        else:
            assert len(calls) > 0, f"Expected {operation} to be called, but was never called"
    
    def assert_not_called(self, operation: str) -> None:
        """断言操作未被调用（用于测试）。
        
        Args:
            operation: 操作名称
            
        Raises:
            AssertionError: 断言失败
        """
        calls = self.get_calls_by_operation(operation)
        assert len(calls) == 0, (
            f"Expected {operation} to not be called, "
            f"but was called {len(calls)} times"
        )
    
    # ========== 生命周期方法 ==========
    
    async def initialize(self) -> None:
        """初始化 Adapter（可选覆盖）。
        
        在首次使用前调用，用于初始化 SDK 客户端等资源。
        """
        if self._initialized:
            return
        await self._on_initialize()
        self._initialized = True
        logger.debug(f"Adapter 已初始化: {self.name}")
    
    async def cleanup(self) -> None:
        """清理 Adapter 资源（可选覆盖）。
        
        在关闭时调用，用于释放连接、关闭客户端等。
        """
        await self._on_cleanup()
        self._initialized = False
        logger.debug(f"Adapter 已清理: {self.name}")
    
    async def __aenter__(self) -> BaseAdapter:
        """异步上下文管理器入口。"""
        await self.initialize()
        return self
    
    async def __aexit__(self, exc_type, exc_val, exc_tb) -> None:
        """异步上下文管理器出口。"""
        await self.cleanup()
    
    # ========== 内部方法（供装饰器使用） ==========
    
    def _resolve_mode(self, meta: AdapterMethodMeta) -> AdapterMode:
        """解析操作的有效模式。
        
        优先级：装饰器 mode_override > 配置 operation_modes > 配置全局 mode
        
        Args:
            meta: 操作元信息
            
        Returns:
            AdapterMode: 有效模式
        """
        # 1. 装饰器强制覆盖
        if meta.mode_override:
            return meta.mode_override
        
        # 2. 配置 per-method 覆盖
        if meta.name in self.settings.method_modes:
            return self.settings.method_modes[meta.name]
        
        # 3. 全局配置
        return self.settings.mode
    
    def _record_call(
        self,
        operation: str,
        args: tuple[Any, ...],
        kwargs: dict[str, Any],
        mode: AdapterMode,
    ) -> CallRecord:
        """记录调用。
        
        Args:
            operation: 操作名称
            args: 位置参数
            kwargs: 关键字参数
            mode: 调用模式
            
        Returns:
            CallRecord: 调用记录
        """
        record = CallRecord(
            operation=operation,
            args=args,
            kwargs=kwargs,
            mode=mode,
        )
        self._call_history.append(record)
        
        logger.debug(
            f"Adapter 调用: {self.name}.{operation} | "
            f"mode={mode} | "
            f"args={args} | "
            f"kwargs={kwargs}"
        )
        
        return record
    
    async def _invoke_mock(
        self,
        meta: AdapterMethodMeta,
        args: tuple[Any, ...],
        kwargs: dict[str, Any],
    ) -> Any:
        """调用 mock 逻辑。
        
        优先级：
        1. 装饰器注册的 mock_handler（带逻辑）
        2. 装饰器的 mock_response（固定值）
        3. 配置的 mock_operation_responses（按操作）
        4. 配置的 mock_default_response（全局）
        5. 根据 mock_strategy 生成默认响应
        
        Args:
            meta: 操作元信息
            args: 位置参数
            kwargs: 关键字参数
            
        Returns:
            Any: mock 响应
        """
        # 模拟延迟
        if self.settings.mock_delay > 0:
            await asyncio.sleep(self.settings.mock_delay)
        
        # 1. 装饰器注册的 mock handler（带逻辑）
        if meta.mock_handler:
            logger.debug(f"使用 mock handler: {self.name}.{meta.name}")
            return await meta.mock_handler(self, *args, **kwargs)
        
        # 2. 装饰器的 mock_response
        if meta.mock_response is not None:
            logger.debug(f"使用装饰器 mock_response: {self.name}.{meta.name}")
            return meta.mock_response
        
        # 3. 配置的按方法 mock 响应
        method_response = self.settings.mock_method_responses.get(meta.name)
        if method_response is not None:
            logger.debug(f"使用配置 mock_method_responses: {self.name}.{meta.name}")
            return method_response
        
        # 4. 配置的全局 mock 响应
        if self.settings.mock_default_response is not None:
            logger.debug(f"使用配置 mock_default_response: {self.name}.{meta.name}")
            return self.settings.mock_default_response
        
        # 5. 根据策略生成默认响应
        return self._generate_mock_response(meta.name, args, kwargs)
    
    def _generate_mock_response(
        self,
        operation: str,
        args: tuple[Any, ...],
        kwargs: dict[str, Any],
    ) -> dict[str, Any]:
        """根据 mock_strategy 生成默认响应。
        
        Args:
            operation: 操作名称
            args: 位置参数
            kwargs: 关键字参数
            
        Returns:
            dict: 默认 mock 响应
        """
        strategy = self.settings.mock_strategy
        
        if strategy == "success":
            return {
                "success": True,
                "mock": True,
                "gateway": self.name,
                "operation": operation,
            }
        
        if strategy == "failure":
            return {
                "success": False,
                "mock": True,
                "gateway": self.name,
                "operation": operation,
                "error": "mock failure",
            }
        
        if strategy == "echo":
            return {
                "success": True,
                "mock": True,
                "gateway": self.name,
                "operation": operation,
                "echo": {
                    "args": args,
                    "kwargs": kwargs,
                },
            }
        
        # noop / custom / 其他
        return {
            "success": True,
            "mock": True,
            "gateway": self.name,
            "operation": operation,
        }
    
    # ========== 钩子方法（子类可覆盖） ==========
    
    async def _on_initialize(self) -> None:
        """初始化钩子（子类可覆盖）。
        
        在 initialize() 中调用，用于初始化 SDK 客户端等。
        """
        pass
    
    async def _on_cleanup(self) -> None:
        """清理钩子（子类可覆盖）。
        
        在 cleanup() 中调用，用于释放资源。
        """
        pass
    
    async def _on_before_call(
        self,
        operation: str,
        args: tuple[Any, ...],
        kwargs: dict[str, Any],
    ) -> None:
        """调用前钩子（子类可覆盖）。
        
        在真实调用前触发，可用于：
        - 参数验证
        - 日志记录
        - 指标采集
        
        Args:
            operation: 操作名称
            args: 位置参数
            kwargs: 关键字参数
        """
        pass
    
    async def _on_after_call(
        self,
        operation: str,
        args: tuple[Any, ...],
        kwargs: dict[str, Any],
        result: Any,
    ) -> None:
        """调用后钩子（子类可覆盖）。
        
        在真实调用成功后触发，可用于：
        - 响应日志
        - 指标采集
        - 结果缓存
        
        Args:
            operation: 操作名称
            args: 位置参数
            kwargs: 关键字参数
            result: 调用结果
        """
        pass
    
    async def _on_call_error(
        self,
        operation: str,
        args: tuple[Any, ...],
        kwargs: dict[str, Any],
        error: Exception,
    ) -> None:
        """调用异常钩子（子类可覆盖）。
        
        在真实调用抛出异常时触发，可用于：
        - 异常日志
        - 告警
        - 指标采集
        
        注意：此钩子不会吞掉异常，异常会继续向上抛出。
        
        Args:
            operation: 操作名称
            args: 位置参数
            kwargs: 关键字参数
            error: 异常对象
        """
        logger.error(
            f"Adapter 调用异常: {self.name}.{operation} | "
            f"error={type(error).__name__}: {error}"
        )
    
    def __repr__(self) -> str:
        """字符串表示。"""
        return (
            f"<{self.__class__.__name__} "
            f"name={self.name!r} "
            f"mode={self.settings.mode!r} "
            f"enabled={self.settings.enabled}>"
        )


__all__ = [
    "BaseAdapter",
    "CallRecord",
]



"""UPSERT 策略实现。

为不同数据库提供 UPSERT 操作的策略实现。
"""

from __future__ import annotations

from abc import ABC, abstractmethod
from typing import Any, ClassVar

from sqlalchemy import Table, text
from sqlalchemy.ext.asyncio import AsyncSession

from aury.boot.common.logging import logger


class UpsertStrategy(ABC):
    """UPSERT 策略抽象基类。
    
    为不同数据库提供 UPSERT 操作的统一接口。
    """
    
    @abstractmethod
    async def execute(
        self,
        session: AsyncSession,
        table: Table,
        data_list: list[dict[str, Any]],
        index_elements: list[str],
        update_columns: list[str],
    ) -> None:
        """执行 UPSERT 操作。
        
        Args:
            session: 数据库会话
            table: 表对象
            data_list: 数据字典列表
            index_elements: 索引字段列表
            update_columns: 需要更新的字段列表
        """
        pass


class PostgreSQLUpsertStrategy(UpsertStrategy):
    """PostgreSQL UPSERT 策略。
    
    使用 ON CONFLICT DO UPDATE 语法。
    """
    
    async def execute(
        self,
        session: AsyncSession,
        table: Table,
        data_list: list[dict[str, Any]],
        index_elements: list[str],
        update_columns: list[str],
    ) -> None:
        """执行 PostgreSQL UPSERT。"""
        from sqlalchemy.dialects.postgresql import insert as pg_insert
        
        stmt = pg_insert(table).values(data_list)
        stmt = stmt.on_conflict_do_update(
            index_elements=index_elements,
            set_={col: stmt.excluded[col] for col in update_columns}
        )
        
        await session.execute(stmt)
        await session.flush()
        logger.debug(f"批量插入或更新 {len(data_list)} 条记录（PostgreSQL）")


class SQLiteUpsertStrategy(UpsertStrategy):
    """SQLite UPSERT 策略。
    
    使用 ON CONFLICT DO UPDATE 语法（SQLite 3.24.0+）。
    """
    
    async def execute(
        self,
        session: AsyncSession,
        table: Table,
        data_list: list[dict[str, Any]],
        index_elements: list[str],
        update_columns: list[str],
    ) -> None:
        """执行 SQLite UPSERT。"""
        from sqlalchemy.dialects.sqlite import insert as sqlite_insert
        
        stmt = sqlite_insert(table).values(data_list)
        stmt = stmt.on_conflict_do_update(
            index_elements=index_elements,
            set_={col: stmt.excluded[col] for col in update_columns}
        )
        
        await session.execute(stmt)
        await session.flush()
        logger.debug(f"批量插入或更新 {len(data_list)} 条记录（SQLite）")


class MySQLUpsertStrategy(UpsertStrategy):
    """MySQL UPSERT 策略。
    
    使用 ON DUPLICATE KEY UPDATE 语法。
    """
    
    async def execute(
        self,
        session: AsyncSession,
        table: Table,
        data_list: list[dict[str, Any]],
        index_elements: list[str],
        update_columns: list[str],
    ) -> None:
        """执行 MySQL UPSERT。"""
        # 构建批量 INSERT ... ON DUPLICATE KEY UPDATE 语句
        columns = list(data_list[0].keys())
        column_names = ", ".join(columns)
        
        # 构建 VALUES 子句（批量插入）
        values_placeholders = []
        params = {}
        for idx, data in enumerate(data_list):
            row_placeholders = []
            for col in columns:
                param_name = f"{col}_{idx}"
                row_placeholders.append(f":{param_name}")
                params[param_name] = data[col]
            values_placeholders.append(f"({', '.join(row_placeholders)})")
        
        values_clause = ", ".join(values_placeholders)
        
        # 构建 ON DUPLICATE KEY UPDATE 子句
        update_clauses = [f"{col} = VALUES({col})" for col in update_columns]
        update_clause = ", ".join(update_clauses)
        
        # 执行 SQL
        sql = f"INSERT INTO {table.name} ({column_names}) VALUES {values_clause} ON DUPLICATE KEY UPDATE {update_clause}"
        await session.execute(text(sql), params)
        await session.flush()
        logger.debug(f"批量插入或更新 {len(data_list)} 条记录（MySQL）")


class UpsertStrategyFactory:
    """UPSERT 策略工厂。
    
    根据数据库方言创建相应的策略实例。
    """
    
    _strategies: ClassVar[dict[str, type[UpsertStrategy]]] = {
        "postgresql": PostgreSQLUpsertStrategy,
        "sqlite": SQLiteUpsertStrategy,
        "mysql": MySQLUpsertStrategy,
    }
    
    @classmethod
    def create(cls, dialect_name: str) -> UpsertStrategy:
        """创建 UPSERT 策略实例。
        
        Args:
            dialect_name: 数据库方言名称
            
        Returns:
            UpsertStrategy: 策略实例
            
        Raises:
            NotImplementedError: 如果数据库不支持 UPSERT 操作
        """
        strategy_class = cls._strategies.get(dialect_name)
        if strategy_class is None:
            raise NotImplementedError(
                f"数据库 {dialect_name} 不支持 UPSERT 操作。"
                "请使用 bulk_insert() 或实现自定义的插入/更新逻辑。"
            )
        
        return strategy_class()
    
    @classmethod
    def register(cls, dialect_name: str, strategy_class: type[UpsertStrategy]) -> None:
        """注册自定义策略。
        
        Args:
            dialect_name: 数据库方言名称
            strategy_class: 策略类
        """
        cls._strategies[dialect_name] = strategy_class
        logger.debug(f"注册 UPSERT 策略: {dialect_name} -> {strategy_class.__name__}")


__all__ = [
    "MySQLUpsertStrategy",
    "PostgreSQLUpsertStrategy",
    "SQLiteUpsertStrategy",
    "UpsertStrategy",
    "UpsertStrategyFactory",
]






"""错误处理链管理器。

提供错误处理链的构建和管理。
"""

from fastapi import Request
from fastapi.responses import JSONResponse

from .handlers import (
    BaseErrorHandler,
    DatabaseErrorHandler,
    ErrorHandler,
    HTTPExceptionHandler,
    ServiceErrorHandler,
    ValidationErrorHandler,
)


class ErrorHandlerChain:
    """错误处理链管理器。"""
    
    def __init__(self) -> None:
        """初始化处理链。"""
        self._chain = self._build_chain()
    
    def _build_chain(self) -> ErrorHandler:
        """构建处理链。
        
        Returns:
            ErrorHandler: 处理链头部
        """
        # 按优先级顺序构建处理链
        # 1. BaseErrorHandler - 处理应用层自定义异常
        # 2. ServiceErrorHandler - 处理 Domain 层 ServiceException（业务异常）
        # 3. HTTPExceptionHandler - 处理 FastAPI HTTP 异常
        # 4. ValidationErrorHandler - 处理 Pydantic 验证异常
        # 5. DatabaseErrorHandler - 处理数据库异常（包括 Domain 层其他异常）
        base_handler = BaseErrorHandler()
        service_handler = ServiceErrorHandler()
        http_handler = HTTPExceptionHandler()
        validation_handler = ValidationErrorHandler()
        db_handler = DatabaseErrorHandler()
        
        base_handler.set_next(service_handler).set_next(http_handler).set_next(validation_handler).set_next(db_handler)
        
        return base_handler
    
    async def handle(self, exception: Exception, request: Request) -> JSONResponse:
        """处理异常。
        
        Args:
            exception: 异常对象
            request: 请求对象
            
        Returns:
            JSONResponse: 响应对象
        """
        return await self._chain.process(exception, request)


# 全局异常处理器实例
error_handler_chain = ErrorHandlerChain()


async def global_exception_handler(request: Request, exc: Exception) -> JSONResponse:
    """全局异常处理器（FastAPI集成）。
    
    使用方式:
        app.add_exception_handler(Exception, global_exception_handler)
    """
    return await error_handler_chain.handle(exc, request)


__all__ = [
    "ErrorHandlerChain",
    "error_handler_chain",
    "global_exception_handler",
]





"""数据库事务管理工具。

提供多种事务管理方式：
1. @transactional - 通用装饰器，自动从参数中获取session
2. @requires_transaction - Repository 事务检查装饰器
3. transactional_context - 上下文管理器（支持 on_commit 回调）
4. TransactionManager - 手动控制事务（支持 Savepoints）
"""

from __future__ import annotations

from collections.abc import AsyncGenerator, Awaitable, Callable
from contextlib import asynccontextmanager
import contextvars
from functools import wraps
from inspect import signature
from typing import Any

from sqlalchemy.ext.asyncio import AsyncSession

from aury.boot.common.logging import logger
from aury.boot.domain.exceptions import TransactionRequiredError

# 用于跟踪嵌套事务的上下文变量
_transaction_depth: contextvars.ContextVar[int] = contextvars.ContextVar("transaction_depth", default=0)

# 用于存储 on_commit 回调的上下文变量（不设置 default，避免可变对象问题）
_on_commit_callbacks: contextvars.ContextVar[list[Callable[[], Any] | Callable[[], Awaitable[Any]]] | None] = (
    contextvars.ContextVar("on_commit_callbacks", default=None)
)


def on_commit(callback: Callable[[], Any] | Callable[[], Awaitable[Any]]) -> None:
    """注册事务提交后执行的回调函数。
    
    类似 Django 的 transaction.on_commit()，回调会在事务成功提交后执行。
    如果事务回滚，回调不会执行。
    
    Args:
        callback: 回调函数，可以是同步或异步函数
    
    用法:
        async with transactional_context(session):
            order = await repo.create({...})
            on_commit(lambda: send_notification.delay(order.id))
            # 只有事务提交成功后，才会执行 send_notification
    
    注意:
        - 回调在事务提交后、上下文退出前执行
        - 如果回调抛出异常，不会影响事务（事务已提交）
        - 嵌套事务中注册的回调，只在最外层事务提交后执行
    """
    callbacks = _on_commit_callbacks.get()
    if callbacks is None:
        callbacks = []
        _on_commit_callbacks.set(callbacks)
    callbacks.append(callback)


async def _execute_on_commit_callbacks() -> None:
    """执行所有 on_commit 回调。"""
    callbacks = _on_commit_callbacks.get()
    if not callbacks:
        return
    
    # 清空回调列表
    _on_commit_callbacks.set(None)
    
    for callback in callbacks:
        try:
            result = callback()
            # 如果是协程，await 它
            if hasattr(result, "__await__"):
                await result
        except Exception as exc:
            # 回调异常不影响事务（事务已提交），只记录日志
            logger.error(f"on_commit 回调执行失败: {exc}")


@asynccontextmanager
async def transactional_context(session: AsyncSession, auto_commit: bool = True) -> AsyncGenerator[AsyncSession]:
    """
    事务上下文管理器，自动处理提交和回滚。
    
    Args:
        session: 数据库会话
        auto_commit: 是否自动提交（默认True）
    
    特性:
        - 支持嵌套调用：只有最外层会 commit/rollback
        - 兼容 SQLAlchemy 2.0 autobegin 模式
        - 支持 on_commit 回调
    
    用法:
        async with transactional_context(session):
            await repo1.create(...)
            await repo2.update(...)
            on_commit(lambda: print("事务已提交"))
            # 成功自动提交，异常自动回滚
    """
    # 使用 contextvars 跟踪嵌套深度，避免依赖 in_transaction()
    depth = _transaction_depth.get()
    is_outermost = depth == 0
    _transaction_depth.set(depth + 1)
    
    # 最外层事务初始化回调列表
    if is_outermost:
        _on_commit_callbacks.set(None)
    
    try:
        yield session
        # 只有最外层且 auto_commit=True 时才提交
        if auto_commit and is_outermost:
            await session.commit()
            logger.debug("事务提交成功")
            # 提交成功后执行 on_commit 回调
            await _execute_on_commit_callbacks()
    except Exception as exc:
        # 只有最外层才回滚
        if is_outermost:
            await session.rollback()
            # 回滚时清空回调列表（不执行）
            _on_commit_callbacks.set(None)
            logger.error(f"事务回滚: {exc}")
        raise
    finally:
        _transaction_depth.set(depth)


def transactional[T](func: Callable[..., T]) -> Callable[..., T]:
    """
    通用事务装饰器，自动从函数参数中查找session并管理事务。
    
    支持多种用法：
    1. 参数名为 session 的函数
    2. 参数名为 db 的函数
    3. 类方法中有 self.session 属性
    
    特性：
    - 自动识别session参数
    - 成功时自动提交
    - 异常时自动回滚
    - 支持嵌套（内层事务不会重复提交）
    
    用法示例:
        @transactional
        async def create_user(session: AsyncSession, name: str):
            # 操作会在事务中执行
            user = User(name=name)
            session.add(user)
            await session.flush()
            return user
        
        # 或者在类方法中
        class UserService:
            def __init__(self, session: AsyncSession):
                self.session = session
            
            @transactional
            async def create_with_profile(self, name: str):
                # 自动使用 self.session
                user = await self.create_user(name)
                profile = await self.create_profile(user.id)
                return user, profile
    """

    @wraps(func)
    async def wrapper(*args, **kwargs):
        session: AsyncSession | None = None
        
        # 策略1: 从kwargs中获取session或db
        if "session" in kwargs:
            session = kwargs["session"]
        elif "db" in kwargs:
            session = kwargs["db"]
        else:
            # 策略2: 从args中获取 (检查类型注解)
            sig = signature(func)
            params = list(sig.parameters.keys())
            
            for i, param_name in enumerate(params):
                if i < len(args):
                    param_value = args[i]
                    # 检查是否是AsyncSession类型
                    if isinstance(param_value, AsyncSession):
                        session = param_value
                        break
                    # 检查参数名
                    if param_name in ("session", "db"):
                        session = param_value
                        break
            
            # 策略3: 从self.session获取 (类方法)
            if session is None and args and hasattr(args[0], "session"):
                session = args[0].session

        if session is None:
            raise ValueError(
                f"无法找到session参数。请确保函数 {func.__name__} 有一个名为 'session' 或 'db' 的参数，"
                "或者类有 'session' 属性。"
            )

        # 使用事务上下文管理器，它会自动处理嵌套情况
        logger.debug(f"执行事务 {func.__name__}")
        async with transactional_context(session):
            return await func(*args, **kwargs)

    return wrapper


class TransactionManager:
    """
    事务管理器，提供更细粒度的事务控制。
    
    支持功能：
    - 手动控制事务 begin/commit/rollback
    - Savepoints（保存点）
    - on_commit 回调
    
    用法:
        tm = TransactionManager(session)
        
        await tm.begin()
        try:
            await repo1.create(data)
            
            # 使用 Savepoint
            sid = await tm.savepoint()
            try:
                await repo2.update(data)
            except Exception:
                await tm.savepoint_rollback(sid)
            
            await tm.commit()
        except Exception:
            await tm.rollback()
            raise
    """
    
    def __init__(self, session: AsyncSession):
        self.session = session
        self._transaction = None
        self._savepoints: dict[str, Any] = {}  # savepoint_id -> nested transaction
        self._savepoint_counter = 0
        self._on_commit_callbacks: list[Callable[[], Any] | Callable[[], Awaitable[Any]]] = []
    
    async def begin(self):
        """开始事务。"""
        if self._transaction is None:
            self._transaction = await self.session.begin()
            logger.debug("手动开启事务")
    
    async def commit(self):
        """提交事务。"""
        if self._transaction:
            await self.session.commit()
            self._transaction = None
            logger.debug("手动提交事务")
            # 执行 on_commit 回调
            await self._execute_callbacks()
    
    async def rollback(self):
        """回滚事务。"""
        if self._transaction:
            await self.session.rollback()
            self._transaction = None
            self._savepoints.clear()
            self._on_commit_callbacks.clear()  # 回滚时清空回调
            logger.debug("手动回滚事务")
    
    async def savepoint(self, name: str | None = None) -> str:
        """创建保存点。
        
        Args:
            name: 保存点名称，可选。如果不提供，自动生成。
        
        Returns:
            str: 保存点 ID，用于后续 commit 或 rollback
        
        用法:
            sid = await tm.savepoint()
            try:
                await risky_operation()
                await tm.savepoint_commit(sid)
            except Exception:
                await tm.savepoint_rollback(sid)
        """
        self._savepoint_counter += 1
        savepoint_id = name or f"sp_{self._savepoint_counter}"
        
        # 使用 begin_nested() 创建 savepoint
        nested = await self.session.begin_nested()
        self._savepoints[savepoint_id] = nested
        logger.debug(f"创建保存点: {savepoint_id}")
        return savepoint_id
    
    async def savepoint_commit(self, savepoint_id: str) -> None:
        """提交保存点。
        
        Args:
            savepoint_id: savepoint() 返回的 ID
        """
        if savepoint_id not in self._savepoints:
            logger.warning(f"保存点不存在: {savepoint_id}")
            return
        
        # SQLAlchemy 的 begin_nested() 在退出时自动提交
        # 这里只需要从字典中移除
        del self._savepoints[savepoint_id]
        logger.debug(f"确认保存点: {savepoint_id}")
    
    async def savepoint_rollback(self, savepoint_id: str) -> None:
        """回滚到保存点。
        
        Args:
            savepoint_id: savepoint() 返回的 ID
        """
        if savepoint_id not in self._savepoints:
            logger.warning(f"保存点不存在: {savepoint_id}")
            return
        
        nested = self._savepoints.pop(savepoint_id)
        await nested.rollback()
        logger.debug(f"回滚到保存点: {savepoint_id}")
    
    def on_commit(self, callback: Callable[[], Any] | Callable[[], Awaitable[Any]]) -> None:
        """注册事务提交后执行的回调。
        
        Args:
            callback: 回调函数，可以是同步或异步函数
        """
        self._on_commit_callbacks.append(callback)
    
    async def _execute_callbacks(self) -> None:
        """执行所有 on_commit 回调。"""
        callbacks = self._on_commit_callbacks
        self._on_commit_callbacks = []
        
        for callback in callbacks:
            try:
                result = callback()
                if hasattr(result, "__await__"):
                    await result
            except Exception as exc:
                logger.error(f"on_commit 回调执行失败: {exc}")
    
    async def __aenter__(self):
        await self.begin()
        return self
    
    async def __aexit__(self, exc_type, exc_val, exc_tb):
        if exc_type is None:
            await self.commit()
        else:
            await self.rollback()


def ensure_transaction(session: AsyncSession) -> bool:
    """
    检查当前会话是否在事务中。
    
    用于在Repository中进行安全检查。
    """
    return session.in_transaction()


def requires_transaction(func: Callable) -> Callable:
    """事务必需装饰器。
    
    确保方法在事务中执行，如果不在事务中则抛出 TransactionRequiredError。
    
    用于 Repository 方法，强制要求在事务中调用。
    
    用法:
        class UserRepository(BaseRepository):
            @requires_transaction
            async def update_user(self, user, data):
                # 此方法必须在事务中执行
                return await self.update(user, data)
    """
    
    @wraps(func)
    async def wrapper(self, *args, **kwargs):
        if not self._session.in_transaction():
            raise TransactionRequiredError(
                f"方法 {func.__name__} 需要在事务中执行，但当前会话不在事务中"
            )
        return await func(self, *args, **kwargs)
    
    return wrapper


def isolated_task[T](func: Callable[..., T]) -> Callable[..., T]:
    """后台任务隔离装饰器。
    
    重置事务上下文，避免从父协程继承 _transaction_depth 导致 auto_commit 失效。
    
    问题背景：
        asyncio.create_task() 会继承父协程的 contextvars。如果父协程在 @transactional 中，
        _transaction_depth > 0，子任务的 auto_commit 和 transactional_context 都会认为
        "在事务中" 而跳过 commit，导致 session 关闭时 rollback。
    
    用法：
        @isolated_task
        async def upload_cover(space_id: int, cover_url: str):
            async with db.session() as session:
                async with transactional_context(session):
                    repo = SpaceRepository(session, Space)
                    space = await repo.get(space_id)
                    await repo.update(space, {"cover": cover_url})
                # 现在会正常 commit
        
        # 在 Service 中 spawn
        asyncio.create_task(upload_cover(space.id, url))
    """
    @wraps(func)
    async def wrapper(*args, **kwargs):
        # 重置事务深度，让当前任务成为独立的事务上下文
        token = _transaction_depth.set(0)
        try:
            return await func(*args, **kwargs)
        finally:
            _transaction_depth.reset(token)
    
    return wrapper


@asynccontextmanager
async def isolated_context() -> AsyncGenerator[None]:
    """后台任务隔离上下文管理器。
    
    与 @isolated_task 作用相同，但用于上下文管理器形式。
    
    用法：
        async def background_job():
            async with isolated_context():
                async with db.session() as session:
                    async with transactional_context(session):
                        ...
    """
    token = _transaction_depth.set(0)
    try:
        yield
    finally:
        _transaction_depth.reset(token)


__all__ = [
    "TransactionManager",
    "TransactionRequiredError",
    "_transaction_depth",  # 内部使用，不对外文档化
    "ensure_transaction",
    "isolated_context",
    "isolated_task",
    "on_commit",
    "requires_transaction",
    "transactional",
    "transactional_context",
]




"""通知器基类。

定义通知器接口，所有通知器实现都应继承此类。
"""

from __future__ import annotations

from abc import ABC, abstractmethod
from typing import TYPE_CHECKING

if TYPE_CHECKING:
    from ..events import AlertNotification


class AlertNotifier(ABC):
    """告警通知器基类。
    
    所有通知器都应继承此类并实现 send 方法。
    
    示例：
        class MyNotifier(AlertNotifier):
            def __init__(self, api_key: str):
                self.api_key = api_key
            
            @classmethod
            def from_config(cls, config: dict) -> "MyNotifier":
                return cls(api_key=config["api_key"])
            
            async def send(self, notification: AlertNotification) -> bool:
                # 发送通知
                ...
    """
    
    @classmethod
    @abstractmethod
    def from_config(cls, config: dict) -> "AlertNotifier":
        """从配置字典创建通知器实例。
        
        Args:
            config: 配置字典（从环境变量解析）
        
        Returns:
            通知器实例
        """
        ...
    
    @abstractmethod
    async def send(self, notification: "AlertNotification") -> bool:
        """发送告警通知。
        
        Args:
            notification: 告警通知对象
        
        Returns:
            bool: 是否发送成功
        """
        ...


__all__ = ["AlertNotifier"]



"""模型基类和类型装饰器。

提供 SQLAlchemy 2.0 声明式基类和跨数据库类型装饰器。
"""

from __future__ import annotations

from typing import TYPE_CHECKING
from uuid import UUID

from sqlalchemy import String
from sqlalchemy.orm import DeclarativeBase
from sqlalchemy.types import TypeDecorator

from aury.boot.common.logging import logger

if TYPE_CHECKING:
    pass


class GUID(TypeDecorator):
    """跨数据库 UUID 类型装饰器。

    自动适配不同数据库：
    - PostgreSQL: 使用原生 UUID 类型（需要安装 [pg] 可选依赖）
    - 其他数据库: 使用 CHAR(36) 存储字符串格式的 UUID
    """

    impl = String(36)
    cache_ok = True

    def load_dialect_impl(self, dialect):
        if dialect.name == "postgresql":
            # PostgreSQL: 使用原生 UUID 类型
            try:
                from sqlalchemy.dialects.postgresql import UUID as PGUUID
                return dialect.type_descriptor(PGUUID(as_uuid=True))
            except ImportError:
                # 如果未安装 PostgreSQL 支持，回退到字符串
                logger.warning(
                    "PostgreSQL UUID 支持需要安装 [pg] 可选依赖："
                    "pip install aury-boot[pg]"
                )
                return dialect.type_descriptor(String(36))
        else:
            # 其他数据库: 使用字符串
            return dialect.type_descriptor(String(36))

    def process_bind_param(self, value, dialect):
        if value is None:
            return value
        if not isinstance(value, UUID):
            raise TypeError(f"Expected UUID, got {type(value)}")
        return str(value)

    def process_result_value(self, value, dialect):
        if value is None:
            return value
        if isinstance(value, UUID):
            return value
        return UUID(value)


class Base(DeclarativeBase):
    """SQLAlchemy 2.0 声明式基类"""

    # 自动生成表名逻辑（可选，例如将 UserProfile 转为 user_profile）
    # pass




"""HTTP 请求日志中间件。

提供 HTTP 相关的日志功能，包括：
- 请求日志中间件（支持链路追踪）
- 请求日志装饰器
"""

from __future__ import annotations

from collections.abc import Callable
from functools import wraps
import time
import uuid

from starlette.middleware.base import BaseHTTPMiddleware
from starlette.requests import Request
from starlette.responses import Response

from aury.boot.application.errors.chain import global_exception_handler
from aury.boot.common.logging import get_trace_id, logger, set_trace_id


def _record_exception_to_span(exc: Exception) -> None:
    """将异常记录到当前 OTEL span（使用与 loguru 一致的格式）。"""
    try:
        from opentelemetry import trace
        
        from aury.boot.common.logging.format import format_exception_compact
        
        span = trace.get_current_span()
        if span and span.is_recording():
            # 使用与 loguru 一致的堆栈格式（包含代码行和局部变量）
            formatted_tb = format_exception_compact(
                type(exc), exc, exc.__traceback__
            )
            
            # 记录异常，并将格式化堆栈放入 attributes
            span.record_exception(exc, attributes={
                "exception.stacktrace": formatted_tb,
            })
            span.set_status(trace.Status(trace.StatusCode.ERROR, str(exc)))
    except ImportError:
        pass  # OTEL 未安装
    except Exception:
        pass  # 忽略记录错误


def log_request[T](func: Callable[..., T]) -> Callable[..., T]:
    """请求日志装饰器。
    
    记录请求的详细信息。
    
    使用示例:
        @router.get("/users")
        @log_request
        async def get_users(request: Request):
            return {"users": []}
    """
    @wraps(func)
    async def wrapper(*args, **kwargs) -> T:
        request: Request | None = None
        
        # 查找Request对象
        for arg in args:
            if isinstance(arg, Request):
                request = arg
                break
        
        if not request:
            request = kwargs.get("request")
        
        # 记录请求信息
        if request:
            logger.info(
                f"请求: {request.method} {request.url.path} | "
                f"客户端: {request.client.host if request.client else 'unknown'} | "
                f"查询参数: {dict(request.query_params)}"
            )
        
        try:
            # 执行函数
            start_time = time.time()
            response = await func(*args, **kwargs)
            duration = time.time() - start_time
            
            # 记录响应信息
            if request:
                logger.info(
                    f"响应: {request.method} {request.url.path} | "
                    f"耗时: {duration:.3f}s"
                )
            
            return response
        except Exception as exc:
            # 记录错误
            if request:
                logger.error(
                    f"错误: {request.method} {request.url.path} | "
                    f"异常: {type(exc).__name__}: {exc}"
                )
            raise
    
    return wrapper


# 请求/响应体最大记录长度
MAX_BODY_LOG_SIZE = 2000

# 预热 uuid/os.urandom，避免首次调用时的初始化开销
_ = uuid.uuid4()
# 不记录 body 的 Content-Type
SKIP_BODY_CONTENT_TYPES = (
    "multipart/form-data",
    "application/octet-stream",
    "image/",
    "audio/",
    "video/",
)


def _truncate_body(body: bytes | None, max_size: int = MAX_BODY_LOG_SIZE) -> str | None:
    """截取请求/响应体用于日志记录。"""
    if not body:
        return None
    try:
        text = body.decode("utf-8")
        if len(text) > max_size:
            return text[:max_size] + f"...(截取，总长{len(text)})"
        return text
    except UnicodeDecodeError:
        return f"<二进制数据 {len(body)} bytes>"


def _should_log_body(content_type: str | None) -> bool:
    """判断是否应该记录 body。"""
    if not content_type:
        return True
    content_type = content_type.lower()
    return all(skip_type not in content_type for skip_type in SKIP_BODY_CONTENT_TYPES)


class RequestLoggingMiddleware(BaseHTTPMiddleware):
    """请求日志中间件（支持链路追踪和告警）。
    
    自动记录所有HTTP请求的详细信息，包括：
    - 请求方法、路径、查询参数、请求体
    - 客户端IP、User-Agent
    - 响应状态码、耗时、响应体
    - 链路追踪 ID（X-Trace-ID / X-Request-ID）
    - 慢请求和异常告警（如果启用告警系统）
    
    注意：文件上传、二进制数据等不会记录 body 内容。
    
    使用示例:
        from aury.boot.application.middleware.logging import RequestLoggingMiddleware
        
        app.add_middleware(RequestLoggingMiddleware, slow_request_threshold=1.0)
    """
    
    def __init__(self, app, slow_request_threshold: float = 1.0) -> None:
        """初始化中间件。
        
        Args:
            app: ASGI 应用
            slow_request_threshold: 慢请求阈值（秒），默认 1.0
        """
        super().__init__(app)
        self.slow_request_threshold = slow_request_threshold
    
    async def dispatch(self, request: Request, call_next) -> Response:
        """处理请求并记录日志。"""
        start_time = time.time()
        
        # 从请求头获取或生成链路追踪 ID
        trace_id = (
            request.headers.get("x-trace-id") or
            request.headers.get("x-request-id") or
            str(uuid.uuid4())
        )
        set_trace_id(trace_id)
        
        # 获取客户端信息
        client_host = request.client.host if request.client else "unknown"
        content_type = request.headers.get("content-type", "")
        
        # 读取请求体
        request_body: bytes | None = None
        request_body_log: str | None = None
        if request.method in ("POST", "PUT", "PATCH", "DELETE"):
            try:
                request_body = await request.body()
                if _should_log_body(content_type):
                    request_body_log = _truncate_body(request_body)
                else:
                    request_body_log = f"<{content_type}, {len(request_body)} bytes>"
            except Exception:
                pass
        
        # 构建请求日志
        request_log = f"→ {request.method} {request.url.path}"
        if request.query_params:
            request_log += f" | 参数: {dict(request.query_params)}"
        if request_body_log:
            request_log += f" | Body: {request_body_log}"
        request_log += f" | 客户端: {client_host} | Trace-ID: {trace_id}"
        
        logger.info(request_log)
        
        # 执行请求
        try:
            response = await call_next(request)
            duration = time.time() - start_time
            
            # 在响应头中添加追踪 ID
            response.headers["x-trace-id"] = trace_id
            
            # 记录响应信息
            status_code = response.status_code
            log_level = "error" if status_code >= 500 else "warning" if status_code >= 400 else "info"
            
            response_log = (
                f"← {request.method} {request.url.path} | "
                f"状态: {status_code} | "
                f"耗时: {duration:.3f}s | "
                f"Trace-ID: {trace_id}"
            )
            logger.log(log_level.upper(), response_log)
            
            # 写入 access 日志（简洁格式）
            logger.bind(access=True).info(
                f"{request.method} {request.url.path} {status_code} {duration:.3f}s"
            )
            
            # 慢请求警告
            if duration > self.slow_request_threshold:
                logger.warning(
                    f"慢请求: {request.method} {request.url.path} | "
                    f"耗时: {duration:.3f}s (阈值: {self.slow_request_threshold}s) | "
                    f"Trace-ID: {trace_id}"
                )
            
            return response
            
        except Exception as exc:
            duration = time.time() - start_time
            # diagnose=True 会自动记录局部变量（request_body_log, client_host, trace_id 等）
            logger.exception(
                f"请求处理失败: {request.method} {request.url.path} | "
                f"耗时: {duration:.3f}s | Trace-ID: {trace_id}"
            )
            
            # 将异常记录到当前 OTEL span（以便告警系统提取）
            _record_exception_to_span(exc)
            
            # 使用全局异常处理器生成响应，而不是直接抛出异常
            # BaseHTTPMiddleware 中直接 raise 会绕过 FastAPI 的异常处理器
            response = await global_exception_handler(request, exc)
            response.headers["x-trace-id"] = trace_id
            return response


class WebSocketLoggingMiddleware:
    """WebSocket 日志中间件。
    
    记录 WebSocket 连接生命周期和消息收发（可选）。
    
    使用示例:
        from aury.boot.application.middleware.logging import WebSocketLoggingMiddleware
        
        app.add_middleware(WebSocketLoggingMiddleware, log_messages=True)
    """
    
    def __init__(
        self,
        app,
        *,
        log_messages: bool = False,
        max_message_length: int = 500,
    ) -> None:
        """初始化 WebSocket 日志中间件。
        
        Args:
            app: ASGI 应用
            log_messages: 是否记录消息内容（默认 False，注意性能和敏感数据）
            max_message_length: 消息内容最大记录长度
        """
        self.app = app
        self.log_messages = log_messages
        self.max_message_length = max_message_length
    
    async def __call__(self, scope, receive, send) -> None:
        if scope["type"] != "websocket":
            await self.app(scope, receive, send)
            return
        
        # 获取或生成 trace_id
        headers = dict(scope.get("headers", []))
        trace_id = (
            headers.get(b"x-trace-id", b"").decode() or
            headers.get(b"x-request-id", b"").decode() or
            str(uuid.uuid4())
        )
        set_trace_id(trace_id)
        
        path = scope.get("path", "/")
        client = scope.get("client")
        client_host = f"{client[0]}:{client[1]}" if client else "unknown"
        
        start_time = time.time()
        message_count = {"sent": 0, "received": 0}
        
        async def logging_receive():
            message = await receive()
            msg_type = message.get("type", "")
            
            if msg_type == "websocket.connect":
                logger.info(
                    f"WS → 连接建立: {path} | "
                    f"客户端: {client_host} | Trace-ID: {trace_id}"
                )
            elif msg_type == "websocket.disconnect":
                duration = time.time() - start_time
                logger.info(
                    f"WS ← 连接关闭: {path} | "
                    f"时长: {duration:.1f}s | "
                    f"收/发: {message_count['received']}/{message_count['sent']} | "
                    f"Trace-ID: {trace_id}"
                )
            elif msg_type == "websocket.receive":
                message_count["received"] += 1
                if self.log_messages:
                    text = message.get("text") or message.get("bytes", b"").decode("utf-8", errors="replace")
                    if len(text) > self.max_message_length:
                        text = text[:self.max_message_length] + "..."
                    logger.debug(f"WS → 收到: {path} | {text}")
            
            return message
        
        async def logging_send(message):
            msg_type = message.get("type", "")
            
            if msg_type == "websocket.send":
                message_count["sent"] += 1
                if self.log_messages:
                    text = message.get("text") or message.get("bytes", b"").decode("utf-8", errors="replace")
                    if len(text) > self.max_message_length:
                        text = text[:self.max_message_length] + "..."
                    logger.debug(f"WS ← 发送: {path} | {text}")
            elif msg_type == "websocket.close":
                code = message.get("code", 1000)
                reason = message.get("reason", "")
                duration = time.time() - start_time
                log_level = "warning" if code != 1000 else "info"
                logger.log(
                    log_level.upper(),
                    f"WS ← 服务关闭: {path} | "
                    f"Code: {code} | 原因: {reason or '正常'} | "
                    f"时长: {duration:.1f}s | Trace-ID: {trace_id}"
                )
            
            await send(message)
        
        try:
            await self.app(scope, logging_receive, logging_send)
        except Exception as exc:
            duration = time.time() - start_time
            logger.exception(
                f"WS ✖ 异常: {path} | "
                f"时长: {duration:.1f}s | "
                f"收/发: {message_count['received']}/{message_count['sent']} | "
                f"Trace-ID: {trace_id}"
            )
            raise


__all__ = [
    "RequestLoggingMiddleware",
    "WebSocketLoggingMiddleware",
    "log_request",
]





"""第三方接口适配器装饰器。

提供 @adapter_method 装饰器，用于声明式定义 Adapter 方法和挡板（Mock）逻辑。

核心特性：
- 声明方法名称，用于配置层按方法覆盖模式（method_modes）
- 支持链式 .mock 注册挡板处理函数
- 自动根据 settings.mode 路由到真实调用或挡板逻辑
- 记录调用历史，便于测试断言

挡板定义方式：
1. @method.mock 装饰器：定义带逻辑的挡板函数，可以根据参数返回不同结果
2. mock_response 参数：简单场景直接指定固定响应值
3. 配置 mock_method_responses：按方法名配置固定响应
4. 配置 mock_default_response：全局默认响应

使用示例:
    class PaymentAdapter(HttpAdapter):
        # 方法 1：使用 @method.mock 定义挡板逻辑
        @adapter_method("create")
        async def create_order(self, amount: int, order_id: str) -> dict:
            # 真实调用逻辑
            return await self._request("POST", "/charge", json={...})
        
        @create_order.mock
        async def create_order_mock(self, amount: int, order_id: str) -> dict:
            # 挡板逻辑，可以模拟各种场景
            if amount > 10000:
                return {"success": False, "error": "金额超限"}  # 模拟失败
            return {"success": True, "mock": True}  # 模拟成功
        
        # 方法 2：简单场景直接指定固定响应
        @adapter_method("query", mock_response={"status": "paid", "mock": True})
        async def query_order(self, order_id: str) -> dict:
            return await self._request("GET", f"/orders/{order_id}")
"""

from __future__ import annotations

import asyncio
from dataclasses import dataclass, field
from functools import wraps
from typing import TYPE_CHECKING, Any, Awaitable, Callable, TypeVar

from aury.boot.common.logging import logger

from .config import AdapterMode
from .exceptions import AdapterDisabledError

if TYPE_CHECKING:
    from .base import BaseAdapter

T = TypeVar("T")
F = TypeVar("F", bound=Callable[..., Awaitable[Any]])


@dataclass
class AdapterMethodMeta:
    """适配器方法元信息。
    
    存储 @adapter_method 装饰器声明的信息，包括：
    - 方法名称
    - mock 处理函数
    - 固定 mock 响应
    - 模式覆盖
    """
    
    name: str
    mock_handler: Callable[..., Awaitable[Any]] | None = None
    mock_response: Any = None
    mode_override: AdapterMode | None = None
    
    # 内部使用：原始真实方法
    _real_method: Callable[..., Awaitable[Any]] | None = field(default=None, repr=False)


class AdapterMethodDescriptor:
    """适配器方法描述符。
    
    实现类似 @property 的描述符协议，支持链式 .mock 注册。
    
    使用示例:
        class PaymentAdapter(HttpAdapter):
            @adapter_method("create")
            async def create_order(self, ...): ...
            
            @create_order.mock
            async def create_order_mock(self, ...): ...
    """
    
    def __init__(
        self,
        real_method: Callable[..., Awaitable[Any]],
        meta: AdapterMethodMeta,
    ) -> None:
        """初始化描述符。
        
        Args:
            real_method: 真实调用方法
            meta: 方法元信息
        """
        self._real_method = real_method
        self._meta = meta
        self._meta._real_method = real_method
        
        # 复制原方法的文档和名称
        self.__name__ = real_method.__name__
        self.__doc__ = real_method.__doc__
        self.__module__ = real_method.__module__
        self.__qualname__ = getattr(real_method, "__qualname__", real_method.__name__)
    
    def __get__(self, obj: BaseAdapter | None, objtype: type | None = None) -> Any:
        """描述符协议：获取绑定方法。"""
        if obj is None:
            # 类访问，返回描述符本身（用于 .mock 链式调用）
            return self
        
        # 实例访问，返回包装后的绑定方法
        return self._create_bound_method(obj)
    
    def __set_name__(self, owner: type, name: str) -> None:
        """描述符协议：设置属性名。"""
        self._attr_name = name
    
    def _create_bound_method(self, adapter: BaseAdapter) -> Callable[..., Awaitable[Any]]:
        """创建绑定到实例的方法。"""
        meta = self._meta
        real_method = self._real_method
        
        @wraps(real_method)
        async def bound_method(*args: Any, **kwargs: Any) -> Any:
            # 解析当前操作的有效模式
            mode = adapter._resolve_mode(meta)
            
            # 记录调用
            adapter._record_call(
                operation=meta.name,
                args=args,
                kwargs=kwargs,
                mode=mode,
            )
            
            # 检查是否禁用
            if not adapter.settings.enabled or mode == "disabled":
                raise AdapterDisabledError(
                    f"{adapter.name}.{meta.name} is disabled in current environment",
                    adapter_name=adapter.name,
                    operation=meta.name,
                )
            
            # mock 模式
            if mode == "mock":
                return await adapter._invoke_mock(meta, args, kwargs)
            
            # real / sandbox 模式：调用原始方法
            # 触发钩子
            await adapter._on_before_call(meta.name, args, kwargs)
            
            try:
                result = await real_method(adapter, *args, **kwargs)
                await adapter._on_after_call(meta.name, args, kwargs, result)
                return result
            except Exception as exc:
                await adapter._on_call_error(meta.name, args, kwargs, exc)
                raise
        
        return bound_method
    
    def mock(self, mock_method: F) -> F:
        """注册 mock 处理函数（链式装饰器）。
        
        使用示例:
            @adapter_method("create")
            async def create_order(self, ...): ...
            
            @create_order.mock
            async def create_order_mock(self, ...): ...
        
        Args:
            mock_method: mock 处理函数
            
        Returns:
            原始 mock 方法（保持可调用）
        """
        self._meta.mock_handler = mock_method
        logger.debug(f"Adapter 注册 mock handler: {self.__name__} -> {mock_method.__name__}")
        
        # 返回原方法，使其可以作为独立方法调用（如果需要）
        return mock_method


def adapter_method(
    name: str,
    *,
    mock_response: Any = None,
    mode_override: AdapterMode | None = None,
) -> Callable[[F], AdapterMethodDescriptor]:
    """适配器方法装饰器。
    
    声明一个 Adapter 方法，支持：
    - 根据配置自动切换 real/mock 模式
    - 链式 .mock 注册 mock 处理函数
    - 调用历史记录
    
    Args:
        name: 方法名称，用于配置层按方法覆盖模式
        mock_response: 简单 mock 场景的固定响应（可选）
        mode_override: 强制覆盖模式，不走配置（可选，用于某些方法必须真实调用的场景）
    
    使用示例:
        # 基础用法
        @adapter_method("create")
        async def create_order(self, amount: int) -> dict:
            return await self._request("POST", "/charge", json={"amount": amount})
        
        # 带固定 mock 响应
        @adapter_method("query", mock_response={"status": "paid", "mock": True})
        async def query_order(self, order_id: str) -> dict:
            return await self._request("GET", f"/orders/{order_id}")
        
        # 强制走真实（无论配置如何）
        @adapter_method("health", mode_override="real")
        async def health_check(self) -> dict:
            return await self._request("GET", "/health")
        
        # 链式 mock
        @adapter_method("create")
        async def create_order(self, amount: int) -> dict:
            return await self._request("POST", "/charge", json={"amount": amount})
        
        @create_order.mock
        async def create_order_mock(self, amount: int) -> dict:
            if amount > 10000:
                return {"success": False, "error": "超限"}
            return {"success": True, "mock": True, "amount": amount}
    
    Returns:
        装饰器函数
    """
    def decorator(fn: F) -> AdapterMethodDescriptor:
        meta = AdapterMethodMeta(
            name=name,
            mock_response=mock_response,
            mode_override=mode_override,
        )
        
        descriptor = AdapterMethodDescriptor(fn, meta)
        logger.debug(f"Adapter 注册方法: {fn.__name__} -> {name}")
        
        return descriptor
    
    return decorator


__all__ = [
    "AdapterMethodDescriptor",
    "AdapterMethodMeta",
    "adapter_method",
]



"""告警规则定义。

定义告警规则数据结构和匹配逻辑。
"""

from __future__ import annotations

import fnmatch
import re
from dataclasses import dataclass, field
from typing import TYPE_CHECKING

from .events import AlertEventType, AlertSeverity

if TYPE_CHECKING:
    from .events import AlertEvent


@dataclass
class AlertRule:
    """告警规则。
    
    定义何时触发告警、如何聚合、发送到哪些通知器。
    
    示例：
        # 慢请求规则：1分钟内5次触发，5分钟抑制
        rule = AlertRule(
            name="slow_request",
            event_types=[AlertEventType.SLOW_REQUEST],
            threshold=1.0,
            aggregate_window=60,
            aggregate_threshold=5,
            suppress_seconds=300,
            notifiers=["feishu"],
        )
        
        # 关键接口规则：更严格的阈值
        rule = AlertRule(
            name="critical_api",
            event_types=[AlertEventType.SLOW_REQUEST],
            path_pattern="/api/v1/payments/*",
            threshold=0.5,
            aggregate_threshold=1,
            notifiers=["feishu", "sms"],
        )
    """
    
    name: str
    event_types: list[AlertEventType]
    
    # 触发条件
    threshold: float | None = None          # 慢阈值（秒），仅对 slow_* 类型有效
    severity_min: AlertSeverity = AlertSeverity.WARNING
    
    # 过滤条件
    source_filter: str | None = None        # api / task / scheduler
    path_pattern: str | None = None         # 路径匹配（支持 * 通配符）
    exclude_paths: list[str] | None = None  # 排除路径列表（支持 * 通配符）
    
    # 聚合配置
    aggregate_window: int = 10              # 滑动窗口（秒）
    aggregate_threshold: int = 1            # 触发阈值
    suppress_seconds: int = 300             # 抑制时间（秒）
    
    # 通知配置
    notifiers: list[str] = field(default_factory=lambda: ["default"])
    
    # 编译后的正则（内部使用）
    _path_regex: re.Pattern | None = field(default=None, repr=False)
    _exclude_regexes: list[re.Pattern] = field(default_factory=list, repr=False)
    
    def __post_init__(self) -> None:
        """初始化后编译路径正则。"""
        if self.path_pattern:
            # 将通配符转换为正则
            regex_pattern = fnmatch.translate(self.path_pattern)
            self._path_regex = re.compile(regex_pattern)
        
        if self.exclude_paths:
            # 编译所有排除路径的正则
            for exclude_pattern in self.exclude_paths:
                regex_pattern = fnmatch.translate(exclude_pattern)
                self._exclude_regexes.append(re.compile(regex_pattern))
    
    def matches(self, event: "AlertEvent") -> bool:
        """检查事件是否匹配规则。
        
        Args:
            event: 告警事件
        
        Returns:
            bool: 是否匹配
        """
        # 检查事件类型
        if event.event_type not in self.event_types:
            return False
        
        # 检查严重级别
        severity_order = [AlertSeverity.INFO, AlertSeverity.WARNING, AlertSeverity.ERROR, AlertSeverity.CRITICAL]
        if severity_order.index(event.severity) < severity_order.index(self.severity_min):
            return False
        
        # 检查来源
        if self.source_filter and event.source != self.source_filter:
            return False
        
        # 检查路径
        if self._path_regex:
            endpoint = event.metadata.get("endpoint", "")
            if not self._path_regex.fullmatch(endpoint):
                return False
        
        # 检查排除路径
        if self._exclude_regexes:
            endpoint = event.metadata.get("endpoint", "")
            for exclude_regex in self._exclude_regexes:
                if exclude_regex.fullmatch(endpoint):
                    return False  # 匹配到排除规则，不触发告警
        
        # 检查阈值（对于 slow_* 类型）
        if self.threshold is not None and event.event_type in (
            AlertEventType.SLOW_REQUEST,
            AlertEventType.SLOW_SQL,
        ):
            duration = event.metadata.get("duration", 0)
            if duration < self.threshold:
                return False
        
        return True


def load_rules_from_dict(data: dict) -> tuple[dict, list[AlertRule]]:
    """从字典加载规则配置。
    
    Args:
        data: 规则配置字典（通常从 YAML 加载）
    
    Returns:
        (defaults, rules) 元组
    """
    defaults = data.get("defaults", {})
    rules = []
    
    for rule_data in data.get("rules", []):
        # 解析事件类型
        event_types = []
        for et in rule_data.get("event_types", []):
            if isinstance(et, str):
                event_types.append(AlertEventType(et))
            else:
                event_types.append(et)
        
        # 解析严重级别
        severity_min = rule_data.get("severity_min", defaults.get("severity_min", "warning"))
        if isinstance(severity_min, str):
            severity_min = AlertSeverity(severity_min.lower())
        
        rule = AlertRule(
            name=rule_data["name"],
            event_types=event_types,
            threshold=rule_data.get("threshold", defaults.get("threshold")),
            severity_min=severity_min,
            source_filter=rule_data.get("source_filter") or rule_data.get("source"),
            path_pattern=rule_data.get("path_pattern"),
            exclude_paths=rule_data.get("exclude_paths"),
            aggregate_window=rule_data.get("aggregate_window", defaults.get("aggregate_window", 10)),
            aggregate_threshold=rule_data.get("aggregate_threshold", defaults.get("aggregate_threshold", 1)),
            suppress_seconds=rule_data.get("suppress_seconds", defaults.get("suppress_seconds", 300)),
            notifiers=rule_data.get("notifiers", ["default"]),
        )
        rules.append(rule)
    
    return defaults, rules


__all__ = [
    "AlertRule",
    "load_rules_from_dict",
]



"""分页和排序标准化模块。

提供统一的分页和排序参数定义，以及分页结果封装。
使用 Pydantic 2.5 进行数据验证和序列化。
"""

from __future__ import annotations

from typing import Any

from pydantic import BaseModel, Field, field_validator


class PaginationParams(BaseModel):
    """分页参数。
    
    使用 offset/limit 模式，与 SQL 语义一致。
    提供 `of()` 方法支持 page/size 风格的输入。
    
    示例:
        # 方式1: offset/limit（底层/API风格）
        params = PaginationParams(offset=20, limit=10)
        
        # 方式2: page/size（UI风格）
        params = PaginationParams.of(page=3, size=10)
    """
    
    offset: int = Field(default=0, ge=0, description="偏移量")
    limit: int = Field(default=20, ge=1, le=100, description="每页记录数，最大100")
    
    @classmethod
    def of(cls, page: int = 1, size: int = 20) -> "PaginationParams":
        """从 page/size 构造分页参数。
        
        Args:
            page: 页码，从 1 开始
            size: 每页记录数
            
        Returns:
            PaginationParams: 分页参数对象
            
        Raises:
            ValueError: 当 page < 1 或 size 无效时
        """
        if page < 1:
            raise ValueError("页码必须大于 0")
        return cls(offset=(page - 1) * size, limit=size)
    
    @property
    def page(self) -> int:
        """当前页码（从 1 开始）。"""
        if self.limit == 0:
            return 1
        return self.offset // self.limit + 1
    
    @property
    def size(self) -> int:
        """每页记录数（limit 的别名）。"""
        return self.limit


class SortParams(BaseModel):
    """排序参数。
    
    定义排序的字段和方向。
    
    支持两种语法：
    - 简洁语法: "-created_at" (前缀 - 表示降序)
    - 完整语法: "created_at:desc"
    
    示例:
        # 从字符串解析
        sort_params = SortParams.from_string("-created_at,priority")
        sort_params = SortParams.from_string("created_at:desc,priority:asc")
        
        # 带白名单验证
        sort_params = SortParams.from_string(
            "-created_at",
            allowed_fields={"id", "created_at", "priority"}
        )
        
        # 程序化构建
        sort_params = SortParams(sorts=[("-created_at",), ("priority", "asc")])
    """
    
    sorts: list[tuple[str, str]] = Field(default_factory=list, description="排序字段列表")
    
    @classmethod
    def from_string(
        cls,
        sort_str: str | None,
        *,
        allowed_fields: set[str] | None = None,
        default_direction: str = "asc",
    ) -> SortParams:
        """从字符串解析排序参数。
        
        支持两种语法：
        - 简洁语法: "-created_at" (前缀 - 表示降序)
        - 完整语法: "created_at:desc"
        
        Args:
            sort_str: 排序字符串，如 "-created_at,priority" 或 "created_at:desc,priority:asc"
            allowed_fields: 允许的字段白名单（可选，用于安全校验）
            default_direction: 默认排序方向（当不指定方向时使用）
            
        Returns:
            SortParams: 排序参数对象
            
        Raises:
            ValueError: 字段不在白名单中 或 方向无效
            
        示例:
            >>> SortParams.from_string("-created_at,priority")
            SortParams(sorts=[('created_at', 'desc'), ('priority', 'asc')])
            
            >>> SortParams.from_string("created_at:desc,priority:asc")
            SortParams(sorts=[('created_at', 'desc'), ('priority', 'asc')])
            
            >>> SortParams.from_string("-id", allowed_fields={"id", "name"})
            SortParams(sorts=[('id', 'desc')])
            
            >>> SortParams.from_string("-invalid", allowed_fields={"id", "name"})
            ValueError: 不允许的排序字段: invalid，允许的字段: id, name
        """
        if not sort_str:
            return cls(sorts=[])
        
        sorts = []
        for part in sort_str.split(","):
            part = part.strip()
            if not part:
                continue
            
            # 解析字段和方向
            if part.startswith("-"):
                # 简洁语法: -created_at
                field = part[1:]
                direction = "desc"
            elif ":" in part:
                # 完整语法: created_at:desc
                field, direction = part.split(":", 1)
            else:
                # 无方向指示，使用默认
                field = part
                direction = default_direction
            
            field = field.strip()
            direction = direction.strip().lower()
            
            # 字段白名单校验
            if allowed_fields and field not in allowed_fields:
                allowed_list = ", ".join(sorted(allowed_fields))
                raise ValueError(f"不允许的排序字段: {field}，允许的字段: {allowed_list}")
            
            # 方向校验
            if direction not in ("asc", "desc"):
                raise ValueError(f"排序方向必须是 'asc' 或 'desc'，得到: {direction}")
            
            sorts.append((field, direction))
        
        return cls(sorts=sorts)
    
    def add_sort(self, field: str, direction: str = "asc") -> None:
        """添加排序字段。
        
        Args:
            field: 字段名
            direction: 排序方向，asc 或 desc
        """
        if direction.lower() not in ("asc", "desc"):
            raise ValueError("排序方向必须是 'asc' 或 'desc'")
        self.sorts.append((field, direction.lower()))
    
    def add_sorts(self, *sorts: tuple[str, str]) -> None:
        """添加多个排序字段。
        
        Args:
            *sorts: 排序字段元组列表
        """
        for field, direction in sorts:
            self.add_sort(field, direction)
    
    @field_validator("sorts", mode="before")
    @classmethod
    def validate_sorts(cls, v: Any) -> list[tuple[str, str]]:
        """验证排序字段列表。
        
        Args:
            v: 排序字段列表
            
        Returns:
            list[tuple[str, str]]: 验证后的排序字段列表
        """
        if not v:
            return []
        
        result = []
        for item in v:
            if isinstance(item, str):
                # 如果是字符串，解析为字段和方向
                if item.startswith("-"):
                    field = item[1:]
                    direction = "desc"
                else:
                    field = item
                    direction = "asc"
                result.append((field, direction))
            elif isinstance(item, list | tuple) and len(item) == 2:
                field, direction = item
                if direction.lower() not in ("asc", "desc"):
                    raise ValueError(f"排序方向必须是 'asc' 或 'desc'，得到: {direction}")
                result.append((field, direction.lower()))
            else:
                raise ValueError(f"无效的排序参数: {item}")
        
        return result


class PaginationResult[ModelType](BaseModel):
    """分页结果。
    
    封装分页查询的结果，包括数据列表和分页信息。
    同时提供 offset/limit 和 page/size 两种风格的字段。
    """
    
    items: list[ModelType] = Field(description="数据列表")
    total: int = Field(ge=0, description="总记录数")
    offset: int = Field(ge=0, description="当前偏移量")
    limit: int = Field(ge=1, description="每页记录数")
    has_next: bool = Field(description="是否有下一页")
    has_prev: bool = Field(description="是否有上一页")
    
    @property
    def page(self) -> int:
        """当前页码（从 1 开始）。"""
        if self.limit == 0:
            return 1
        return self.offset // self.limit + 1
    
    @property
    def size(self) -> int:
        """每页记录数（limit 的别名）。"""
        return self.limit
    
    @property
    def total_pages(self) -> int:
        """总页数。"""
        if self.limit == 0:
            return 0
        return (self.total + self.limit - 1) // self.limit
    
    @classmethod
    def create(
        cls,
        items: list[ModelType],
        total: int,
        pagination: PaginationParams,
    ) -> "PaginationResult[ModelType]":
        """创建分页结果。
        
        Args:
            items: 数据列表
            total: 总记录数
            pagination: 分页参数
            
        Returns:
            PaginationResult[ModelType]: 分页结果
        """
        has_next = pagination.offset + pagination.limit < total
        has_prev = pagination.offset > 0
        
        return cls(
            items=items,
            total=total,
            offset=pagination.offset,
            limit=pagination.limit,
            has_next=has_next,
            has_prev=has_prev,
        )
    
    def get_next_params(self) -> PaginationParams | None:
        """获取下一页的分页参数。"""
        if not self.has_next:
            return None
        return PaginationParams(offset=self.offset + self.limit, limit=self.limit)
    
    def get_prev_params(self) -> PaginationParams | None:
        """获取上一页的分页参数。"""
        if not self.has_prev:
            return None
        new_offset = max(0, self.offset - self.limit)
        return PaginationParams(offset=new_offset, limit=self.limit)


class CursorPaginationParams(BaseModel):
    """游标分页参数。
    
    基于游标的分页，适用于大数据集和高性能场景。
    """
    
    cursor: str | None = Field(default=None, description="游标，用于定位数据位置")
    limit: int = Field(default=20, ge=1, le=100, description="每页记录数，最大100")
    direction: str = Field(default="next", description="分页方向：next 或 prev")
    
    @field_validator("direction")
    @classmethod
    def validate_direction(cls, v: str) -> str:
        """验证分页方向。
        
        Args:
            v: 分页方向
            
        Returns:
            str: 验证后的分页方向
        """
        if v.lower() not in ("next", "prev"):
            raise ValueError("分页方向必须是 'next' 或 'prev'")
        return v.lower()


class CursorPaginationResult[ModelType](BaseModel):
    """游标分页结果。
    
    封装游标分页查询的结果。
    """
    
    items: list[ModelType] = Field(description="数据列表")
    next_cursor: str | None = Field(description="下一页游标")
    prev_cursor: str | None = Field(description="上一页游标")
    has_next: bool = Field(description="是否有下一页")
    has_prev: bool = Field(description="是否有上一页")
    
    @classmethod
    def create(
        cls,
        items: list[ModelType],
        next_cursor: str | None = None,
        prev_cursor: str | None = None,
        limit: int = 20
    ) -> CursorPaginationResult[ModelType]:
        """创建游标分页结果。
        
        Args:
            items: 数据列表
            next_cursor: 下一页游标
            prev_cursor: 上一页游标
            limit: 每页记录数
            
        Returns:
            CursorPaginationResult[ModelType]: 游标分页结果
        """
        has_next = next_cursor is not None
        has_prev = prev_cursor is not None
        
        return cls(
            items=items,
            next_cursor=next_cursor,
            prev_cursor=prev_cursor,
            has_next=has_next,
            has_prev=has_prev,
        )


__all__ = [
    "CursorPaginationParams",
    "CursorPaginationResult",
    "PaginationParams",
    "PaginationResult",
    "SortParams",
]



"""Redis 缓存后端实现。

支持普通 Redis 和 Redis Cluster：
- redis://... - 普通 Redis
- redis-cluster://... - Redis Cluster
"""

from __future__ import annotations

import asyncio
import json
import pickle
import time
from collections.abc import Callable
from datetime import timedelta
from typing import TYPE_CHECKING, Any
from urllib.parse import urlparse

from redis.asyncio import Redis

from aury.boot.common.logging import logger

from .base import ICache

if TYPE_CHECKING:
    from redis.asyncio.cluster import RedisCluster
    from aury.boot.infrastructure.clients.redis import RedisClient


class RedisCache(ICache):
    """Redis缓存实现。
    
    支持两种初始化方式：
    1. 传入 URL 自行创建连接（支持 redis:// 和 redis-cluster://）
    2. 传入 RedisClient 实例（推荐）
    """
    
    def __init__(
        self,
        url: str | None = None,
        *,
        redis_client: RedisClient | None = None,
        serializer: str = "json",
    ):
        """初始化Redis缓存。
        
        Args:
            url: Redis连接URL（支持 redis:// 和 redis-cluster://）
            redis_client: RedisClient 实例（推荐）
            serializer: 序列化方式（json/pickle）
        """
        self._url = url
        self._redis_client = redis_client
        self._serializer = serializer
        self._redis: Redis | RedisCluster | None = None
        self._owns_connection = False
        self._is_cluster = False
    
    async def initialize(self) -> None:
        """初始化连接。"""
        # 优先使用 RedisClient
        if self._redis_client is not None:
            self._redis = self._redis_client.connection
            self._is_cluster = getattr(self._redis_client, "is_cluster", False)
            self._owns_connection = False
            mode = "Cluster" if self._is_cluster else "Standalone"
            logger.info(f"Redis缓存初始化成功（使用 RedisClient, {mode}）")
            return
        
        # 使用 URL 创建连接
        if self._url:
            try:
                if self._url.startswith("redis-cluster://"):
                    await self._init_cluster()
                else:
                    await self._init_standalone()
                await self._redis.ping()
                self._owns_connection = True
                mode = "Cluster" if self._is_cluster else "Standalone"
                logger.info(f"Redis缓存初始化成功 ({mode})")
            except Exception as exc:
                logger.error(f"Redis连接失败: {exc}")
                raise
        else:
            raise ValueError("Redis缓存需要提供 url 或 redis_client 参数")
    
    async def _init_standalone(self) -> None:
        """初始化普通 Redis。"""
        self._redis = Redis.from_url(
            self._url,
            encoding="utf-8",
            decode_responses=False,
            socket_connect_timeout=5,
            socket_timeout=5,
        )
        self._is_cluster = False
    
    async def _init_cluster(self) -> None:
        """初始化 Redis Cluster（使用 redis-py）。
        
        支持 URL 格式:
        - redis-cluster://password@host:port （密码在用户名位置）
        - redis-cluster://:password@host:port （标准格式）
        - redis-cluster://username:password@host:port （ACL 模式）
        """
        from redis.asyncio.cluster import RedisCluster
        
        # 转换 URL scheme
        redis_url = self._url.replace("redis-cluster://", "redis://")
        
        # 处理 password@host 格式（转换为标准 :password@host 格式）
        parsed = urlparse(redis_url)
        if parsed.username and not parsed.password:
            # redis://password@host -> redis://:password@host
            redis_url = redis_url.replace(
                f"redis://{parsed.username}@",
                f"redis://:{parsed.username}@"
            )
        
        self._redis = RedisCluster.from_url(
            redis_url,
            decode_responses=False,
            socket_connect_timeout=30,
            socket_timeout=30,
        )
        self._is_cluster = True
    
    async def get(self, key: str, default: Any = None) -> Any:
        """获取缓存。"""
        if not self._redis:
            return default
        
        try:
            data = await self._redis.get(key)
            if data is None:
                return default
            
            # 使用函数式编程处理序列化器
            deserializers: dict[str, Callable[[bytes], Any]] = {
                "json": lambda d: json.loads(d.decode()),
                "pickle": pickle.loads,
            }
            
            deserializer = deserializers.get(self._serializer)
            if deserializer:
                return deserializer(data)
            return data.decode()
        except Exception as exc:
            logger.error(f"Redis获取失败: {key}, {exc}")
            return default
    
    async def set(
        self,
        key: str,
        value: Any,
        expire: int | timedelta | None = None,
    ) -> bool:
        """设置缓存。"""
        if not self._redis:
            return False
        
        try:
            # 使用函数式编程处理序列化器
            serializers: dict[str, Callable[[Any], bytes]] = {
                "json": lambda v: json.dumps(v).encode(),
                "pickle": pickle.dumps,
            }
            
            serializer = serializers.get(self._serializer)
            if serializer:
                data = serializer(value)
            else:
                data = str(value).encode()
            
            # 转换过期时间
            if isinstance(expire, timedelta):
                expire = int(expire.total_seconds())
            
            await self._redis.set(key, data, ex=expire)
            return True
        except Exception as exc:
            logger.error(f"Redis设置失败: {key}, {exc}")
            return False
    
    async def delete(self, *keys: str) -> int:
        """删除缓存。"""
        if not self._redis or not keys:
            return 0
        
        try:
            return await self._redis.delete(*keys)
        except Exception as exc:
            logger.error(f"Redis删除失败: {keys}, {exc}")
            return 0
    
    async def exists(self, *keys: str) -> int:
        """检查缓存是否存在。"""
        if not self._redis or not keys:
            return 0
        
        try:
            return await self._redis.exists(*keys)
        except Exception as exc:
            logger.error(f"Redis检查失败: {keys}, {exc}")
            return 0
    
    async def clear(self) -> None:
        """清空所有缓存。"""
        if self._redis:
            await self._redis.flushdb()
            logger.info("Redis缓存已清空")
    
    async def delete_pattern(self, pattern: str) -> int:
        """按模式删除缓存。
        
        Args:
            pattern: 通配符模式，如 "todo:*"
            
        Returns:
            int: 删除的键数量
        """
        if not self._redis:
            return 0
        
        try:
            # 使用 SCAN 遍历匹配的键（比 KEYS 更安全，不会阻塞）
            count = 0
            cursor = 0
            while True:
                cursor, keys = await self._redis.scan(cursor, match=pattern, count=100)
                if keys:
                    count += await self._redis.delete(*keys)
                if cursor == 0:
                    break
            logger.debug(f"按模式删除缓存: {pattern}, 删除 {count} 个键")
            return count
        except Exception as exc:
            logger.error(f"Redis模式删除失败: {pattern}, {exc}")
            return 0
    
    async def close(self) -> None:
        """关闭连接（仅当自己拥有连接时）。"""
        if self._redis and self._owns_connection:
            if self._is_cluster:
                # redis-py cluster 使用 aclose()
                await self._redis.aclose()
            else:
                await self._redis.close()
            logger.info("Redis连接已关闭")
        self._redis = None
    
    @property
    def redis(self) -> Redis | RedisCluster | None:
        """获取Redis客户端。"""
        return self._redis
    
    @property
    def is_cluster(self) -> bool:
        """检查是否为集群模式。"""
        return self._is_cluster
    
    # ==================== 分布式锁 ====================
    # TODO: 后续优化考虑：
    #   - 看门狗（Watchdog）机制：自动续期，防止业务执行超过锁超时导致提前释放
    #   - 可重入锁（Reentrant Lock）
    #   - Redlock 算法（多 Redis 实例）
    
    async def acquire_lock(
        self,
        key: str,
        token: str,
        timeout: int,
        blocking: bool,
        blocking_timeout: float | None,
    ) -> bool:
        """获取 Redis 分布式锁。"""
        if not self._redis:
            return False
        
        start_time = time.monotonic()
        
        while True:
            # SET NX EX 原子操作
            acquired = await self._redis.set(key, token, nx=True, ex=timeout)
            if acquired:
                return True
            
            if not blocking:
                return False
            
            # 检查是否超时
            if blocking_timeout is not None:
                elapsed = time.monotonic() - start_time
                if elapsed >= blocking_timeout:
                    return False
            
            # 短暂等待后重试
            await asyncio.sleep(0.05)
    
    async def release_lock(self, key: str, token: str) -> bool:
        """释放 Redis 锁（Lua 脚本保证原子性）。"""
        if not self._redis:
            return False
        
        # Lua 脚本：只有 token 匹配才删除
        script = """
        if redis.call("get", KEYS[1]) == ARGV[1] then
            return redis.call("del", KEYS[1])
        else
            return 0
        end
        """
        result = await self._redis.eval(script, 1, key, token)
        return bool(result)


__all__ = ["RedisCache"]
